"""
Contrôleur de cycle de vie des environnements éphémères
"""

import asyncio
import logging
import os
import shutil
import subprocess
from pathlib import Path
from typing import List, Optional, Dict

from .models import (
    EphemeralEnvironment,
    EphemeralStatus,
    IsolationLevel,
    OperationResult
)
from .exceptions import (
    EnvironmentCreationException,
    CleanupException,
    IsolationException
)
from ..models import Backend

logger = logging.getLogger(__name__)


class LifecycleController:
    """Contrôleur de cycle de vie des environnements"""
    
    def __init__(self, manager):
        self.manager = manager
        self.process_manager = ProcessManager()
    
    async def create(self, env: EphemeralEnvironment) -> EphemeralEnvironment:
        """Création complète d'un environnement"""
        
        logger.info(f"Creating environment {env.id} with backend {env.backend}")
        
        try:
            # Création du répertoire isolé
            await self._setup_isolated_directory(env)
            
            # Création de l'environnement virtuel
            await self._create_virtual_environment(env)
            
            # Configuration de l'isolation
            await self._setup_isolation(env)
            
            # Configuration des limites de ressources
            await self._setup_resource_limits(env)
            
            env.status = EphemeralStatus.READY
            return env
            
        except Exception as e:
            env.status = EphemeralStatus.FAILED
            raise EnvironmentCreationException(f"Failed to create environment: {e}")
    
    async def cleanup(self, env: EphemeralEnvironment, force: bool = False):
        """Nettoyage complet d'un environnement"""
        
        logger.info(f"Cleaning up environment {env.id} (force={force})")
        
        try:
            # Arrêt des processus
            await self._stop_processes(env, force=force)
            
            # Nettoyage de l'isolation
            await self._cleanup_isolation(env)
            
            # Suppression des fichiers
            await self._cleanup_filesystem(env, force=force)
            
        except Exception as e:
            if not force:
                raise CleanupException(f"Cleanup failed: {e}")
            else:
                logger.warning(f"Force cleanup error (ignored): {e}")
    
    async def execute_command(
        self,
        env: EphemeralEnvironment,
        command: str,
        timeout: Optional[int] = None,
        capture_output: bool = True
    ) -> OperationResult:
        """Exécution d'une commande dans l'environnement"""
        
        if not env.is_active:
            raise RuntimeError(f"Environment {env.id} is not active")
        
        # Mise à jour de l'activité
        env.update_activity()
        env.status = EphemeralStatus.RUNNING
        
        try:
            result = await self.process_manager.run_command(
                env, command, timeout=timeout, capture_output=capture_output
            )
            
            if env.status == EphemeralStatus.RUNNING and result.success:
                env.status = EphemeralStatus.READY
            
            return result
            
        finally:
            if env.status == EphemeralStatus.RUNNING:
                env.status = EphemeralStatus.READY
    
    async def install_packages(
        self,
        env: EphemeralEnvironment,
        packages: List[str],
        upgrade: bool = False
    ) -> OperationResult:
        """Installation de packages dans l'environnement"""
        
        if not packages:
            return OperationResult(0, "", "", 0.0, "")
        
        # Construction de la commande selon le backend
        if env.backend == Backend.UV:
            base_cmd = "uv pip install"
        elif env.backend == Backend.PDM:
            base_cmd = "pdm add"
        elif env.backend == Backend.POETRY:
            base_cmd = "poetry add"
        else:
            base_cmd = "pip install"
        
        if upgrade and env.backend in [Backend.PIP, Backend.UV]:
            base_cmd += " --upgrade"
        
        packages_str = " ".join(packages)
        command = f"{base_cmd} {packages_str}"
        
        result = await self.execute_command(env, command, timeout=300)
        
        if result.success:
            env.packages.extend(packages)
        
        return result
    
    async def _setup_isolated_directory(self, env: EphemeralEnvironment):
        """Configuration du répertoire isolé"""
        
        if not env.storage_path:
            raise EnvironmentCreationException("No storage path allocated")
        
        # Création des répertoires
        env.storage_path.mkdir(parents=True, exist_ok=True)
        
        # Configuration des permissions restrictives
        os.chmod(env.storage_path, 0o700)
        
        # Création de la structure
        dirs_to_create = ["tmp", "logs", "cache"]
        for dir_name in dirs_to_create:
            (env.storage_path / dir_name).mkdir(exist_ok=True)
        
        logger.debug(f"Created isolated directory: {env.storage_path}")
    
    async def _create_virtual_environment(self, env: EphemeralEnvironment):
        """Création optimisée du virtual environment"""
        
        venv_path = env.storage_path / "venv"
        env.venv_path = venv_path
        
        if env.backend == Backend.UV:
            # uv est le plus rapide pour les créations
            cmd = [
                "uv", "venv",
                str(venv_path),
                "--python", env.python_version,
                "--seed"  # Pré-install pip/setuptools
            ]
        elif env.backend == Backend.PDM:
            cmd = [
                "pdm", "venv", "create",
                "--python", env.python_version,
                str(venv_path)
            ]
        elif env.backend == Backend.POETRY:
            # Poetry gère ses propres venvs, on crée un venv standard
            cmd = [
                f"python{env.python_version}",
                "-m", "venv",
                str(venv_path),
                "--upgrade-deps"
            ]
        else:  # Backend.PIP
            cmd = [
                f"python{env.python_version}",
                "-m", "venv",
                str(venv_path),
                "--upgrade-deps"
            ]
        
        # Exécution avec timeout
        try:
            process = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
                cwd=env.storage_path
            )
            
            stdout, stderr = await asyncio.wait_for(
                process.communicate(),
                timeout=60  # Création rapide requise
            )
            
            if process.returncode != 0:
                raise EnvironmentCreationException(
                    f"Failed to create virtual environment: {stderr.decode()}"
                )
            
            logger.debug(f"Created virtual environment: {venv_path}")
            
        except asyncio.TimeoutError:
            raise EnvironmentCreationException("Virtual environment creation timed out")
    
    async def _setup_isolation(self, env: EphemeralEnvironment):
        """Configuration de l'isolation de sécurité"""
        
        if env.isolation_level == IsolationLevel.CONTAINER:
            await self._setup_container_isolation(env)
        elif env.isolation_level == IsolationLevel.NAMESPACE:
            await self._setup_namespace_isolation(env)
        elif env.isolation_level == IsolationLevel.CHROOT:
            await self._setup_chroot_isolation(env)
        else:
            # PROCESS: isolation basique par processus
            await self._setup_process_isolation(env)
    
    async def _setup_process_isolation(self, env: EphemeralEnvironment):
        """Isolation par processus (niveau de base)"""
        # Configuration des variables d'environnement
        pass  # Implémenté dans ProcessManager
    
    async def _setup_container_isolation(self, env: EphemeralEnvironment):
        """Isolation par container Docker/Podman"""
        try:
            # Détection du runtime de container disponible
            container_runtime = await self._detect_container_runtime()
            
            if not container_runtime:
                logger.warning("No container runtime available, falling back to process isolation")
                await self._setup_process_isolation(env)
                return
            
            # Construction de l'image de base
            image_name = f"gestvenv-ephemeral-{env.python_version}"
            await self._ensure_container_image(container_runtime, image_name, env.python_version)
            
            # Configuration du container
            container_config = await self._build_container_config(env, image_name)
            
            # Création du container
            container_id = await self._create_container(container_runtime, container_config)
            env.container_id = container_id
            
            # Démarrage du container
            await self._start_container(container_runtime, container_id)
            
            logger.info(f"Container isolation configured: {container_id[:12]}")
            
        except Exception as e:
            logger.error(f"Container isolation setup failed: {e}")
            # Fallback vers l'isolation par processus
            await self._setup_process_isolation(env)
    
    async def _setup_namespace_isolation(self, env: EphemeralEnvironment):
        """Isolation par namespaces Linux"""
        try:
            import platform
            if platform.system() != 'Linux':
                logger.warning("Namespace isolation only available on Linux, falling back to process isolation")
                await self._setup_process_isolation(env)
                return
            
            # Vérification des capacités de namespaces
            if not await self._check_namespace_support():
                logger.warning("Namespace isolation not supported, falling back to process isolation")
                await self._setup_process_isolation(env)
                return
            
            # Configuration des namespaces
            namespace_config = {
                'pid': True,      # Isolation des processus
                'net': env.resource_limits.network_access,  # Isolation réseau conditionnelle
                'mnt': True,      # Isolation du système de fichiers
                'ipc': True,      # Isolation IPC
                'uts': True,      # Isolation hostname/domainname
                'user': True      # Isolation utilisateur (si supporté)
            }
            
            # Création du script d'isolation
            isolation_script = await self._create_namespace_script(env, namespace_config)
            env.isolation_script = isolation_script
            
            logger.info(f"Namespace isolation configured for {env.id}")
            
        except Exception as e:
            logger.error(f"Namespace isolation setup failed: {e}")
            await self._setup_process_isolation(env)
    
    async def _setup_chroot_isolation(self, env: EphemeralEnvironment):
        """Isolation par chroot jail"""
        try:
            import os
            
            # Vérification des privilèges root
            if os.getuid() != 0:
                logger.warning("Chroot isolation requires root privileges, falling back to process isolation")
                await self._setup_process_isolation(env)
                return
            
            # Création du chroot jail
            chroot_path = env.storage_path / "chroot"
            await self._setup_chroot_environment(chroot_path, env)
            
            # Configuration du chroot
            env.chroot_path = chroot_path
            
            logger.info(f"Chroot isolation configured: {chroot_path}")
            
        except Exception as e:
            logger.error(f"Chroot isolation setup failed: {e}")
            await self._setup_process_isolation(env)
    
    async def _setup_resource_limits(self, env: EphemeralEnvironment):
        """Configuration des limites de ressources"""
        # TODO: Implémentation complète avec cgroups en Phase 2
        pass
    
    async def _stop_processes(self, env: EphemeralEnvironment, force: bool = False):
        """Arrêt des processus de l'environnement"""
        
        if env.pid:
            try:
                if force:
                    os.kill(env.pid, 9)  # SIGKILL
                else:
                    os.kill(env.pid, 15)  # SIGTERM
                    await asyncio.sleep(2)  # Attendre arrêt gracieux
                    try:
                        os.kill(env.pid, 0)  # Vérifier si encore vivant
                        os.kill(env.pid, 9)  # SIGKILL si nécessaire
                    except ProcessLookupError:
                        pass  # Processus déjà terminé
            except ProcessLookupError:
                pass  # Processus déjà terminé
        
        # Nettoyage des processus enfants
        await self.process_manager.cleanup_processes(env)
    
    async def _cleanup_isolation(self, env: EphemeralEnvironment):
        """Nettoyage de l'isolation"""
        
        if env.container_id:
            # Nettoyage du container
            try:
                await asyncio.create_subprocess_exec(
                    "docker", "rm", "-f", env.container_id,
                    stdout=asyncio.subprocess.DEVNULL,
                    stderr=asyncio.subprocess.DEVNULL
                )
            except Exception as e:
                logger.warning(f"Failed to cleanup container {env.container_id}: {e}")
    
    async def _cleanup_filesystem(self, env: EphemeralEnvironment, force: bool = False):
        """Nettoyage du système de fichiers"""
        
        if not env.storage_path or not env.storage_path.exists():
            return
        
        try:
            # Suppression récursive
            shutil.rmtree(env.storage_path, ignore_errors=force)
            logger.debug(f"Cleaned up filesystem: {env.storage_path}")
            
        except Exception as e:
            if not force:
                raise CleanupException(f"Failed to cleanup filesystem: {e}")
            else:
                logger.warning(f"Filesystem cleanup error (ignored): {e}")


class ProcessManager:
    """Gestionnaire de processus pour environnements éphémères"""
    
    def __init__(self):
        self.active_processes: Dict[str, List[asyncio.subprocess.Process]] = {}
    
    async def run_command(
        self,
        env: EphemeralEnvironment,
        command: str,
        timeout: Optional[int] = None,
        capture_output: bool = True
    ) -> OperationResult:
        """Exécution d'une commande avec gestion des processus"""
        
        import time
        start_time = time.time()
        
        # Construction de l'environnement d'exécution
        exec_env = await self._build_execution_environment(env)
        
        # Construction de la commande complète
        full_command = await self._build_command(env, command)
        
        try:
            # Création du processus
            if capture_output:
                process = await asyncio.create_subprocess_shell(
                    full_command,
                    stdout=asyncio.subprocess.PIPE,
                    stderr=asyncio.subprocess.PIPE,
                    env=exec_env,
                    cwd=env.storage_path
                )
            else:
                process = await asyncio.create_subprocess_shell(
                    full_command,
                    env=exec_env,
                    cwd=env.storage_path
                )
            
            # Enregistrement du processus
            env.pid = process.pid
            if env.id not in self.active_processes:
                self.active_processes[env.id] = []
            self.active_processes[env.id].append(process)
            
            # Attente avec timeout
            if timeout:
                stdout, stderr = await asyncio.wait_for(
                    process.communicate(),
                    timeout=timeout
                )
            else:
                stdout, stderr = await process.communicate()
            
            duration = time.time() - start_time
            
            return OperationResult(
                returncode=process.returncode,
                stdout=stdout.decode() if stdout else "",
                stderr=stderr.decode() if stderr else "",
                duration=duration,
                command=command
            )
            
        except asyncio.TimeoutError:
            # Nettoyage en cas de timeout
            if process:
                process.kill()
                await process.wait()
            
            duration = time.time() - start_time
            return OperationResult(
                returncode=-1,
                stdout="",
                stderr="Command timed out",
                duration=duration,
                command=command
            )
        
        finally:
            # Nettoyage du processus de la liste active
            if env.id in self.active_processes and process in self.active_processes[env.id]:
                self.active_processes[env.id].remove(process)
    
    async def _build_execution_environment(self, env: EphemeralEnvironment) -> Dict[str, str]:
        """Construction de l'environnement d'exécution"""
        
        exec_env = os.environ.copy()
        
        if env.venv_path:
            # Configuration du virtual environment
            exec_env["VIRTUAL_ENV"] = str(env.venv_path)
            exec_env["PATH"] = f"{env.venv_path}/bin:{exec_env.get('PATH', '')}"
            
            # Suppression de PYTHONHOME si présent
            exec_env.pop("PYTHONHOME", None)
        
        # Configuration de Python
        exec_env["PYTHONPATH"] = str(env.storage_path)
        exec_env["PYTHONDONTWRITEBYTECODE"] = "1"
        exec_env["PYTHONUNBUFFERED"] = "1"
        
        # Configuration de pip/uv
        exec_env["PIP_CACHE_DIR"] = str(env.storage_path / "cache" / "pip")
        exec_env["UV_CACHE_DIR"] = str(env.storage_path / "cache" / "uv")
        
        # Variables de sécurité
        if env.security_mode.value == "restricted":
            exec_env["HOME"] = str(env.storage_path)
            exec_env["TMPDIR"] = str(env.storage_path / "tmp")
        
        return exec_env
    
    async def _build_command(self, env: EphemeralEnvironment, command: str) -> str:
        """Construction de la commande avec activation du venv"""
        
        if env.venv_path and env.backend != Backend.POETRY:
            # Activation explicite du virtual environment
            activate_script = env.venv_path / "bin" / "activate"
            if activate_script.exists():
                return f"source {activate_script} && {command}"
        
        return command
    
    async def cleanup_processes(self, env: EphemeralEnvironment):
        """Nettoyage de tous les processus d'un environnement"""
        
        if env.id not in self.active_processes:
            return
        
        processes = self.active_processes[env.id][:]
        
        for process in processes:
            try:
                if process.returncode is None:  # Processus encore actif
                    process.terminate()
                    try:
                        await asyncio.wait_for(process.wait(), timeout=5)
                    except asyncio.TimeoutError:
                        process.kill()
                        await process.wait()
            except Exception as e:
                logger.warning(f"Failed to cleanup process: {e}")
        
        # Nettoyage de la liste
        if env.id in self.active_processes:
            del self.active_processes[env.id]
    
    # === MÉTHODES D'ISOLATION ===
    
    async def _detect_container_runtime(self) -> Optional[str]:
        """Détection du runtime de container disponible"""
        runtimes = ['docker', 'podman']
        
        for runtime in runtimes:
            try:
                process = await asyncio.create_subprocess_exec(
                    runtime, '--version',
                    stdout=asyncio.subprocess.PIPE,
                    stderr=asyncio.subprocess.PIPE
                )
                await process.communicate()
                
                if process.returncode == 0:
                    logger.debug(f"Found container runtime: {runtime}")
                    return runtime
                    
            except FileNotFoundError:
                continue
        
        return None
    
    async def _ensure_container_image(self, runtime: str, image_name: str, python_version: str):
        """Assure qu'une image de container existe"""
        # Vérification si l'image existe
        try:
            process = await asyncio.create_subprocess_exec(
                runtime, 'image', 'inspect', image_name,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE
            )
            await process.communicate()
            
            if process.returncode == 0:
                return  # Image existe déjà
                
        except Exception:
            pass
        
        # Construction de l'image
        dockerfile_content = f"""
FROM python:{python_version}-slim

# Installation des outils de base
RUN apt-get update && apt-get install -y \\
    curl \\
    git \\
    && rm -rf /var/lib/apt/lists/*

# Installation d'uv
RUN curl -LsSf https://astral.sh/uv/install.sh | sh
ENV PATH="/root/.cargo/bin:$PATH"

# Configuration des répertoires de travail
WORKDIR /workspace
RUN mkdir -p /workspace/tmp /workspace/cache

# Configuration de l'environnement
ENV PYTHONUNBUFFERED=1
ENV PYTHONDONTWRITEBYTECODE=1

CMD ["/bin/bash"]
"""
        
        # Création d'un contexte temporaire
        import tempfile
        with tempfile.TemporaryDirectory() as temp_dir:
            dockerfile_path = Path(temp_dir) / "Dockerfile"
            dockerfile_path.write_text(dockerfile_content)
            
            # Construction de l'image
            process = await asyncio.create_subprocess_exec(
                runtime, 'build', 
                '-t', image_name,
                str(temp_dir),
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE
            )
            
            stdout, stderr = await process.communicate()
            
            if process.returncode != 0:
                raise EnvironmentCreationException(f"Failed to build container image: {stderr.decode()}")
            
            logger.info(f"Built container image: {image_name}")
    
    async def _build_container_config(self, env: EphemeralEnvironment, image_name: str) -> dict:
        """Construction de la configuration du container"""
        config = {
            'image': image_name,
            'name': f"gestvenv-{env.id[:8]}",
            'volumes': [
                f"{env.storage_path}:/workspace:rw"
            ],
            'environment': {
                'GESTVENV_ENV_ID': env.id,
                'PYTHONPATH': '/workspace',
                'PIP_CACHE_DIR': '/workspace/cache/pip',
                'UV_CACHE_DIR': '/workspace/cache/uv'
            },
            'working_dir': '/workspace',
            'detach': True,
            'remove': True  # Auto-cleanup
        }
        
        # Limites de ressources
        if env.resource_limits.max_memory:
            config['memory'] = f"{env.resource_limits.max_memory}m"
        
        if env.resource_limits.max_cpu_percent:
            config['cpus'] = str(env.resource_limits.max_cpu_percent / 100.0)
        
        # Isolation réseau
        if not env.resource_limits.network_access:
            config['network'] = 'none'
        
        return config
    
    async def _create_container(self, runtime: str, config: dict) -> str:
        """Création du container"""
        cmd = [runtime, 'create']
        
        # Ajout des options
        cmd.extend(['--name', config['name']])
        cmd.extend(['--workdir', config['working_dir']])
        
        if config.get('detach'):
            cmd.append('--detach')
        
        if config.get('remove'):
            cmd.append('--rm')
        
        # Volumes
        for volume in config.get('volumes', []):
            cmd.extend(['-v', volume])
        
        # Variables d'environnement
        for key, value in config.get('environment', {}).items():
            cmd.extend(['-e', f"{key}={value}"])
        
        # Limites de ressources
        if 'memory' in config:
            cmd.extend(['--memory', config['memory']])
        
        if 'cpus' in config:
            cmd.extend(['--cpus', config['cpus']])
        
        if 'network' in config:
            cmd.extend(['--network', config['network']])
        
        # Image et commande
        cmd.append(config['image'])
        cmd.extend(['sleep', 'infinity'])  # Garder le container vivant
        
        # Exécution
        process = await asyncio.create_subprocess_exec(
            *cmd,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE
        )
        
        stdout, stderr = await process.communicate()
        
        if process.returncode != 0:
            raise EnvironmentCreationException(f"Failed to create container: {stderr.decode()}")
        
        container_id = stdout.decode().strip()
        return container_id
    
    async def _start_container(self, runtime: str, container_id: str):
        """Démarrage du container"""
        process = await asyncio.create_subprocess_exec(
            runtime, 'start', container_id,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE
        )
        
        stdout, stderr = await process.communicate()
        
        if process.returncode != 0:
            raise EnvironmentCreationException(f"Failed to start container: {stderr.decode()}")
    
    async def _check_namespace_support(self) -> bool:
        """Vérification du support des namespaces"""
        try:
            # Vérification de la présence d'unshare
            process = await asyncio.create_subprocess_exec(
                'unshare', '--help',
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE
            )
            await process.communicate()
            
            if process.returncode != 0:
                return False
            
            # Vérification des namespaces dans /proc
            namespace_types = ['pid', 'net', 'mnt', 'ipc', 'uts', 'user']
            proc_ns_path = Path('/proc/self/ns')
            
            if not proc_ns_path.exists():
                return False
            
            for ns_type in namespace_types:
                if not (proc_ns_path / ns_type).exists():
                    logger.debug(f"Namespace {ns_type} not supported")
                    return False
            
            return True
            
        except Exception as e:
            logger.debug(f"Namespace support check failed: {e}")
            return False
    
    async def _create_namespace_script(self, env: EphemeralEnvironment, config: dict) -> str:
        """Création du script d'isolation par namespaces"""
        script_path = env.storage_path / "isolation.sh"
        
        # Construction des options unshare
        unshare_options = []
        
        if config.get('pid'):
            unshare_options.append('--pid')
            unshare_options.append('--fork')
        
        if config.get('net') and not env.resource_limits.network_access:
            unshare_options.append('--net')
        
        if config.get('mnt'):
            unshare_options.append('--mount')
        
        if config.get('ipc'):
            unshare_options.append('--ipc')
        
        if config.get('uts'):
            unshare_options.append('--uts')
        
        # Script d'isolation
        script_content = f"""#!/bin/bash
set -e

# Isolation par namespaces
exec unshare {' '.join(unshare_options)} \\
    --mount-proc=/proc \\
    bash -c '
        # Configuration de l'environnement isolé
        export PYTHONPATH="{env.storage_path}"
        export PYTHONUNBUFFERED=1
        export PYTHONDONTWRITEBYTECODE=1
        export PIP_CACHE_DIR="{env.storage_path}/cache/pip"
        export UV_CACHE_DIR="{env.storage_path}/cache/uv"
        export TMPDIR="{env.storage_path}/tmp"
        
        # Changement de répertoire
        cd "{env.storage_path}"
        
        # Exécution de la commande
        exec "$@"
    ' -- "$@"
"""
        
        script_path.write_text(script_content)
        script_path.chmod(0o755)
        
        return str(script_path)
    
    async def _setup_chroot_environment(self, chroot_path: Path, env: EphemeralEnvironment):
        """Configuration de l'environnement chroot"""
        chroot_path.mkdir(parents=True, exist_ok=True)
        
        # Création de la structure de base
        essential_dirs = [
            'bin', 'usr/bin', 'lib', 'lib64', 'usr/lib', 'usr/lib64',
            'etc', 'tmp', 'dev', 'proc', 'sys', 'workspace'
        ]
        
        for dir_name in essential_dirs:
            (chroot_path / dir_name).mkdir(parents=True, exist_ok=True)
        
        # Copie des binaires essentiels
        essential_binaries = [
            '/bin/bash', '/bin/sh', '/usr/bin/python3',
            '/usr/bin/pip3', '/bin/ls', '/bin/cat'
        ]
        
        for binary in essential_binaries:
            binary_path = Path(binary)
            if binary_path.exists():
                dest_path = chroot_path / binary_path.relative_to(Path('/'))
                dest_path.parent.mkdir(parents=True, exist_ok=True)
                
                try:
                    shutil.copy2(binary_path, dest_path)
                except Exception as e:
                    logger.debug(f"Failed to copy {binary}: {e}")
        
        # Configuration des devices de base
        dev_nodes = [
            ('null', 'c', 1, 3),
            ('zero', 'c', 1, 5),
            ('random', 'c', 1, 8),
            ('urandom', 'c', 1, 9)
        ]
        
        for name, dev_type, major, minor in dev_nodes:
            dev_path = chroot_path / 'dev' / name
            try:
                if dev_type == 'c':
                    # Création de device caractère
                    os.mknod(dev_path, 0o666 | 0o020000, os.makedev(major, minor))
            except Exception as e:
                logger.debug(f"Failed to create device {name}: {e}")
        
        logger.info(f"Chroot environment setup completed: {chroot_path}")


"""
Service de gestion des packages pour GestVenv v1.1
"""

import logging
import time
from pathlib import Path
from typing import Dict, List, Optional

from ..core.models import (
    EnvironmentInfo,
    PackageInfo,
    InstallResult,
    SyncResult,
    PyProjectInfo
)
from ..core.exceptions import (
    PackageInstallationError,
    BackendError,
    ValidationError
)
from ..backends.base import PackageBackend
from ..backends.backend_manager import BackendManager

logger = logging.getLogger(__name__)


class PackageService:
    """Service unifié de gestion des packages"""
    
    def __init__(self, backend_manager: BackendManager, cache_service=None):
        self.backend_manager = backend_manager
        self.cache_service = cache_service
        
    def install_package(
        self, 
        env: EnvironmentInfo, 
        package: str, 
        **kwargs
    ) -> InstallResult:
        """Installation avec backend optimal et cache"""
        start_time = time.time()
        
        try:
            # Validation package
            self._validate_package_specification(package)
            
            # Sélection backend
            backend = self._get_backend_for_env(env)
            
            # Tentative cache en mode hors ligne
            if self.cache_service and self.cache_service.is_offline_mode_enabled():
                cache_result = self.cache_service.install_from_cache(env, package)
                if cache_result.success:
                    return InstallResult(
                        success=True,
                        message=f"Package {package} installé depuis le cache",
                        packages_installed=[package],
                        backend_used="cache",
                        execution_time=time.time() - start_time
                    )
            
            # Installation normale
            result = backend.install_package(env.path, package, **kwargs)
            
            if result.success:
                # Mise en cache si activé
                if self.cache_service:
                    self.cache_service.cache_installed_package(env, package)
                
                # Mise à jour environnement
                env.packages = backend.list_packages(env.path)
                env.updated_at = time.time()
                
                return InstallResult(
                    success=True,
                    message=f"Package {package} installé avec succès",
                    packages_installed=[package],
                    backend_used=backend.name,
                    execution_time=time.time() - start_time
                )
            else:
                return InstallResult(
                    success=False,
                    message=f"Échec installation {package}: {result.message}",
                    packages_failed=[package],
                    backend_used=backend.name,
                    execution_time=time.time() - start_time
                )
                
        except Exception as e:
            logger.error(f"Erreur installation {package}: {e}")
            return InstallResult(
                success=False,
                message=f"Erreur installation: {e}",
                packages_failed=[package],
                execution_time=time.time() - start_time
            )
    
    def uninstall_package(self, env: EnvironmentInfo, package: str) -> bool:
        """Désinstalle un package"""
        try:
            backend = self._get_backend_for_env(env)
            success = backend.uninstall_package(env.path, package)
            
            if success:
                # Mise à jour environnement
                env.packages = backend.list_packages(env.path)
                env.updated_at = time.time()
            
            return success
        except Exception as e:
            logger.error(f"Erreur désinstallation {package}: {e}")
            return False
    
    def update_package(self, env: EnvironmentInfo, package: str) -> bool:
        """Met à jour un package"""
        try:
            backend = self._get_backend_for_env(env)
            success = backend.update_package(env.path, package)
            
            if success:
                env.packages = backend.list_packages(env.path)
                env.updated_at = time.time()
            
            return success
        except Exception as e:
            logger.error(f"Erreur mise à jour {package}: {e}")
            return False
    
    def list_packages(self, env: EnvironmentInfo) -> List[PackageInfo]:
        """Liste les packages installés"""
        try:
            backend = self._get_backend_for_env(env)
            return backend.list_packages(env.path)
        except Exception as e:
            logger.error(f"Erreur listage packages: {e}")
            return []
    
    def install_from_requirements(self, env: EnvironmentInfo, req_path: Path) -> bool:
        """Installation depuis requirements.txt"""
        try:
            if not req_path.exists():
                raise FileNotFoundError(f"Fichier requirements.txt introuvable: {req_path}")
            
            backend = self._get_backend_for_env(env)
            
            if hasattr(backend, 'install_from_requirements'):
                success = backend.install_from_requirements(env.path, req_path)
            else:
                # Installation manuelle ligne par ligne
                success = self._manual_requirements_install(env, req_path, backend)
            
            if success:
                env.packages = backend.list_packages(env.path)
                env.updated_at = time.time()
            
            return success
        except Exception as e:
            logger.error(f"Erreur installation requirements: {e}")
            return False
    
    def install_from_pyproject(
        self, 
        env: EnvironmentInfo, 
        pyproject: PyProjectInfo, 
        groups: Optional[List[str]] = None
    ) -> bool:
        """Installation depuis pyproject.toml"""
        try:
            backend = self._get_backend_for_env(env)
            
            # Support natif pyproject.toml
            if hasattr(backend, 'sync_from_pyproject') and pyproject.source_path:
                success = backend.sync_from_pyproject(
                    env.path, 
                    pyproject.source_path, 
                    groups
                )
            else:
                # Installation manuelle des dépendances
                success = self._manual_pyproject_install(env, pyproject, groups, backend)
            
            if success:
                env.packages = backend.list_packages(env.path)
                env.updated_at = time.time()
            
            return success
        except Exception as e:
            logger.error(f"Erreur installation pyproject: {e}")
            return False
    
    def sync_environment(self, env: EnvironmentInfo) -> SyncResult:
        """Synchronise l'environnement avec son pyproject.toml"""
        start_time = time.time()
        
        try:
            if not env.pyproject_info:
                return SyncResult(
                    success=False,
                    message="Aucun pyproject.toml associé",
                    execution_time=time.time() - start_time
                )
            
            backend = self._get_backend_for_env(env)
            
            # Analyse des différences
            current_packages = {pkg.name: pkg.version for pkg in env.packages}
            expected_deps = env.pyproject_info.extract_dependencies()
            expected_packages = {
                dep.split('>=')[0].split('==')[0].split('[')[0]: dep
                for dep in expected_deps
            }
            
            packages_to_add = []
            packages_to_remove = []
            packages_to_update = []
            
            # Packages à ajouter
            for pkg_name, pkg_spec in expected_packages.items():
                if pkg_name not in current_packages:
                    packages_to_add.append(pkg_spec)
            
            # Packages à supprimer (qui ne sont plus dans pyproject.toml)
            for pkg_name in current_packages:
                if pkg_name not in expected_packages:
                    packages_to_remove.append(pkg_name)
            
            # Installation/suppression
            warnings = []
            
            for package in packages_to_add:
                install_result = self.install_package(env, package)
                if not install_result.success:
                    warnings.append(f"Échec installation {package}")
            
            for package in packages_to_remove:
                if not self.uninstall_package(env, package):
                    warnings.append(f"Échec suppression {package}")
            
            return SyncResult(
                success=True,
                message="Synchronisation terminée",
                packages_added=packages_to_add,
                packages_removed=packages_to_remove,
                packages_updated=packages_to_update,
                warnings=warnings,
                execution_time=time.time() - start_time
            )
            
        except Exception as e:
            return SyncResult(
                success=False,
                message=f"Erreur synchronisation: {e}",
                execution_time=time.time() - start_time
            )
    
    def create_lock_file(self, env: EnvironmentInfo) -> Optional[Path]:
        """Crée un fichier de verrouillage"""
        try:
            backend = self._get_backend_for_env(env)
            
            if hasattr(backend, 'create_lock_file') and env.pyproject_info:
                return backend.create_lock_file(env.pyproject_info.source_path)
            
            return None
        except Exception as e:
            logger.error(f"Erreur création lock file: {e}")
            return None
    
    def install_from_lock(self, env: EnvironmentInfo, lock_path: Path) -> bool:
        """Installation depuis fichier de verrouillage"""
        try:
            backend = self._get_backend_for_env(env)
            
            if hasattr(backend, 'install_from_lock'):
                success = backend.install_from_lock(env.path, lock_path)
                
                if success:
                    env.packages = backend.list_packages(env.path)
                    env.updated_at = time.time()
                
                return success
            
            return False
        except Exception as e:
            logger.error(f"Erreur installation lock: {e}")
            return False
    
    def check_outdated_packages(self, env: EnvironmentInfo) -> List[PackageInfo]:
        """Vérifie les packages obsolètes"""
        try:
            # Implémentation simplifiée - peut être améliorée
            # avec vérification réelle des versions sur PyPI
            outdated = []
            
            for package in env.packages:
                # Logique de vérification (à implémenter selon le backend)
                if hasattr(package, 'is_outdated') and package.is_outdated():
                    outdated.append(package)
            
            return outdated
        except Exception as e:
            logger.error(f"Erreur vérification packages obsolètes: {e}")
            return []
    
    def resolve_dependencies(self, packages: List[str]) -> Dict[str, str]:
        """Résout les dépendances"""
        try:
            # Implémentation simplifiée
            # Dans une vraie implémentation, utiliser un resolver de dépendances
            resolved = {}
            for package in packages:
                pkg_name = package.split('>=')[0].split('==')[0].split('[')[0]
                resolved[pkg_name] = package
            
            return resolved
        except Exception as e:
            logger.error(f"Erreur résolution dépendances: {e}")
            return {}
    
    def _get_backend_for_env(self, env: EnvironmentInfo) -> PackageBackend:
        """Récupère le backend pour un environnement"""
        return self.backend_manager.get_backend(env.backend_type.value, env)
    
    def _validate_package_specification(self, package: str) -> None:
        """Valide une spécification de package"""
        if not package or not package.strip():
            raise ValidationError("Spécification de package vide")
        
        # Validation basique pour éviter l'injection de commandes
        dangerous_chars = [';', '|', '&', '`', '$']
        if any(char in package for char in dangerous_chars):
            raise ValidationError(f"Caractères dangereux dans le package: {package}")
    
    def _manual_requirements_install(
        self, 
        env: EnvironmentInfo, 
        req_path: Path, 
        backend: PackageBackend
    ) -> bool:
        """Installation manuelle depuis requirements.txt"""
        try:
            with open(req_path, 'r', encoding='utf-8') as f:
                lines = f.readlines()
            
            for line in lines:
                line = line.strip()
                if line and not line.startswith('#'):
                    result = backend.install_package(env.path, line)
                    if not result.success:
                        logger.warning(f"Échec installation {line}")
            
            return True
        except Exception as e:
            logger.error(f"Erreur installation manuelle requirements: {e}")
            return False
    
    def _manual_pyproject_install(
        self, 
        env: EnvironmentInfo, 
        pyproject: PyProjectInfo, 
        groups: Optional[List[str]],
        backend: PackageBackend
    ) -> bool:
        """Installation manuelle depuis pyproject.toml"""
        try:
            dependencies = pyproject.extract_dependencies(groups)
            
            for dep in dependencies:
                result = backend.install_package(env.path, dep)
                if not result.success:
                    logger.warning(f"Échec installation {dep}")
            
            return True
        except Exception as e:
            logger.error(f"Erreur installation manuelle pyproject: {e}")
            return False


"""
Service pour la gestion des packages
"""

from typing import List, Dict, Any, Optional
from .gestvenv_service import GestVenvService


class PackageService:
    """Service pour gérer les packages dans les environnements"""
    
    def __init__(self):
        self.gestvenv = GestVenvService()
    
    async def list_packages(self, env_id: str) -> List[Dict[str, Any]]:
        """Liste les packages installés dans un environnement"""
        # Pour les tests, retourner une liste factice
        return [
            {"name": "requests", "version": "2.31.0"},
            {"name": "numpy", "version": "1.24.0"}
        ]
    
    async def install_package(self, env_id: str, package_name: str, version: Optional[str] = None) -> Dict[str, Any]:
        """Installe un package dans un environnement"""
        return {
            "success": True,
            "message": f"Package {package_name} installed successfully"
        }
    
    async def uninstall_package(self, env_id: str, package_name: str) -> Dict[str, Any]:
        """Désinstalle un package d'un environnement"""
        return {
            "success": True,
            "message": f"Package {package_name} uninstalled successfully"
        }


"""
Service pour la gestion du cache
"""

from typing import Dict, Any, Optional, List
from pathlib import Path
import json


class CacheService:
    """Service pour gérer le cache des packages"""
    
    def __init__(self):
        self.cache_dir = Path.home() / ".cache" / "gestvenv"
        self.cache_info = {}
    
    async def get_cache_info(self) -> Dict[str, Any]:
        """Récupère les informations du cache"""
        return {
            "size": 1024 * 1024 * 100,  # 100 MB
            "packages": 42,
            "location": str(self.cache_dir)
        }
    
    async def clear_cache(self) -> Dict[str, Any]:
        """Vide le cache"""
        self.cache_info = {}
        return {
            "success": True,
            "message": "Cache cleared successfully"
        }
    
    async def get_cached_packages(self) -> List[Dict[str, Any]]:
        """Liste les packages en cache"""
        return [
            {"name": "requests", "version": "2.31.0", "size": 1024 * 100},
            {"name": "numpy", "version": "1.24.0", "size": 1024 * 1024 * 10}
        ]


"""
Utilitaires de gestion des chemins pour GestVenv v1.1
"""

import os
import shutil
from datetime import datetime
from pathlib import Path
from typing import List, Optional

import logging
logger = logging.getLogger(__name__)


class PathUtils:
    """Utilitaires de gestion des chemins et fichiers"""
    
    @staticmethod
    def ensure_directory(path: Path) -> bool:
        """Assure qu'un répertoire existe"""
        try:
            path.mkdir(parents=True, exist_ok=True)
            return True
        except (OSError, PermissionError) as e:
            logger.error(f"Impossible de créer {path}: {e}")
            return False
    
    @staticmethod
    def safe_remove_directory(path: Path) -> bool:
        """Supprime un répertoire en sécurité"""
        if not path.exists():
            return True
        
        if not path.is_dir():
            logger.error(f"{path} n'est pas un répertoire")
            return False
        
        try:
            # Vérification sécurité
            if not PathUtils._is_safe_to_remove(path):
                logger.error(f"Suppression refusée pour raisons de sécurité: {path}")
                return False
            
            shutil.rmtree(path)
            return True
        except (OSError, PermissionError) as e:
            logger.error(f"Erreur suppression {path}: {e}")
            return False
    
    @staticmethod
    def get_environment_path(name: str, base_path: Path) -> Path:
        """Chemin d'environnement sécurisé"""
        # Nettoyage nom
        safe_name = PathUtils._sanitize_filename(name)
        return base_path / safe_name
    
    @staticmethod
    def find_pyproject_files(directory: Path) -> List[Path]:
        """Trouve tous les fichiers pyproject.toml"""
        pyproject_files = []
        
        if not directory.exists() or not directory.is_dir():
            return pyproject_files
        
        try:
            # Recherche récursive limitée (3 niveaux max)
            for root in [directory] + list(directory.iterdir())[:10]:
                if root.is_dir():
                    pyproject_path = root / "pyproject.toml"
                    if pyproject_path.exists():
                        pyproject_files.append(pyproject_path)
        except (OSError, PermissionError):
            pass
        
        return pyproject_files
    
    @staticmethod
    def backup_file(source: Path, backup_dir: Path) -> Optional[Path]:
        """Sauvegarde un fichier avec timestamp"""
        if not source.exists():
            return None
        
        try:
            PathUtils.ensure_directory(backup_dir)
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            backup_name = f"{source.name}.{timestamp}.backup"
            backup_path = backup_dir / backup_name
            
            shutil.copy2(source, backup_path)
            return backup_path
        except (OSError, PermissionError) as e:
            logger.error(f"Erreur sauvegarde {source}: {e}")
            return None
    
    @staticmethod
    def find_project_root(start_path: Path) -> Optional[Path]:
        """Trouve la racine d'un projet Python"""
        current = start_path.resolve()
        
        project_markers = [
            "pyproject.toml",
            "setup.py",
            "setup.cfg",
            "requirements.txt",
            ".git",
            ".gitignore"
        ]
        
        # Remonte jusqu'à 5 niveaux
        for _ in range(5):
            for marker in project_markers:
                if (current / marker).exists():
                    return current
            
            parent = current.parent
            if parent == current:  # Racine système
                break
            current = parent
        
        return None
    
    @staticmethod
    def resolve_relative_path(base: Path, relative: str) -> Path:
        """Résout un chemin relatif de manière sécurisée"""
        try:
            resolved = (base / relative).resolve()
            
            # Vérification que le chemin reste dans base
            if not str(resolved).startswith(str(base.resolve())):
                raise ValueError("Path traversal détecté")
            
            return resolved
        except (OSError, ValueError) as e:
            logger.error(f"Erreur résolution chemin: {e}")
            raise
    
    @staticmethod
    def copy_directory_tree(source: Path, target: Path) -> bool:
        """Copie un arbre de répertoires"""
        if not source.exists() or not source.is_dir():
            return False
        
        try:
            # Vérifications sécurité
            if not PathUtils._is_safe_copy_operation(source, target):
                return False
            
            # Suppression cible si existe
            if target.exists():
                PathUtils.safe_remove_directory(target)
            
            shutil.copytree(source, target)
            return True
        except (OSError, PermissionError) as e:
            logger.error(f"Erreur copie {source} → {target}: {e}")
            return False
    
    @staticmethod
    def get_size_mb(path: Path) -> float:
        """Taille d'un fichier/répertoire en MB"""
        if not path.exists():
            return 0.0
        
        try:
            if path.is_file():
                return path.stat().st_size / (1024 * 1024)
            elif path.is_dir():
                total_size = 0
                for file_path in path.rglob('*'):
                    if file_path.is_file():
                        total_size += file_path.stat().st_size
                return total_size / (1024 * 1024)
        except (OSError, PermissionError):
            pass
        
        return 0.0
    
    @staticmethod
    def is_empty_directory(path: Path) -> bool:
        """Vérifie si un répertoire est vide"""
        if not path.exists() or not path.is_dir():
            return False
        
        try:
            return not any(path.iterdir())
        except (OSError, PermissionError):
            return False
    
    @staticmethod
    def clean_empty_directories(root_path: Path) -> int:
        """Nettoie les répertoires vides récursivement"""
        if not root_path.exists() or not root_path.is_dir():
            return 0
        
        cleaned_count = 0
        
        try:
            # Parcours en profondeur d'abord
            for path in sorted(root_path.rglob('*'), key=lambda x: -len(x.parts)):
                if path.is_dir() and PathUtils.is_empty_directory(path):
                    try:
                        path.rmdir()
                        cleaned_count += 1
                    except OSError:
                        pass
        except (OSError, PermissionError):
            pass
        
        return cleaned_count
    
    @staticmethod
    def find_files_by_pattern(directory: Path, pattern: str, max_depth: int = 3) -> List[Path]:
        """Trouve fichiers par pattern avec limitation profondeur"""
        files = []
        
        if not directory.exists() or not directory.is_dir():
            return files
        
        try:
            current_depth = 0
            to_process = [(directory, 0)]
            
            while to_process and current_depth <= max_depth:
                current_dir, depth = to_process.pop(0)
                
                if depth > max_depth:
                    continue
                
                for item in current_dir.iterdir():
                    if item.is_file() and item.match(pattern):
                        files.append(item)
                    elif item.is_dir() and depth < max_depth:
                        to_process.append((item, depth + 1))
                        
                current_depth = max(current_depth, depth)
                
        except (OSError, PermissionError):
            pass
        
        return files
    
    # Méthodes privées
    
    @staticmethod
    def _sanitize_filename(filename: str) -> str:
        """Assainit un nom de fichier"""
        # Caractères interdits
        forbidden_chars = '<>:"/\\|?*'
        sanitized = ''.join('_' if c in forbidden_chars else c for c in filename)
        
        # Longueur maximum
        if len(sanitized) > 255:
            sanitized = sanitized[:255]
        
        # Pas de points à la fin
        sanitized = sanitized.rstrip('.')
        
        return sanitized or "default"
    
    @staticmethod
    def _is_safe_to_remove(path: Path) -> bool:
        """Vérifie si un chemin peut être supprimé en sécurité"""
        path_str = str(path.resolve())
        
        # Répertoires système protégés
        protected_paths = [
            '/etc', '/bin', '/usr', '/var', '/root', '/home',
            'C:\\Windows', 'C:\\Program Files', 'C:\\Users'
        ]
        
        for protected in protected_paths:
            if path_str.startswith(protected):
                return False
        
        # Vérification ownership (Unix)
        if hasattr(os, 'getuid'):
            try:
                stat = path.stat()
                if stat.st_uid != os.getuid():
                    return False
            except OSError:
                return False
        
        return True
    
    @staticmethod
    def _is_safe_copy_operation(source: Path, target: Path) -> bool:
        """Vérifie si une opération de copie est sécurisée"""
        try:
            source_resolved = source.resolve()
            target_resolved = target.resolve()
            
            # Éviter copie récursive
            if str(target_resolved).startswith(str(source_resolved)):
                return False
            
            # Vérifier permissions source
            if not os.access(source, os.R_OK):
                return False
            
            # Vérifier permissions cible
            target_parent = target_resolved.parent
            if not os.access(target_parent, os.W_OK):
                return False
            
            return True
        except (OSError, ValueError):
            return False


"""
Gestionnaire TOML optimisé pour GestVenv v1.1

Utilise une approche hybride :
- tomllib (Python 3.11+) ou tomli pour la lecture
- tomlkit pour l'écriture avec préservation du formatage
- Fallback vers implémentation basique si nécessaire
"""

import sys
from pathlib import Path
from typing import Dict, Any, Union, TextIO
import logging

logger = logging.getLogger(__name__)

# Import conditionnel selon la version Python
if sys.version_info >= (3, 11):
    import tomllib
else:
    try:
        import tomli as tomllib
    except ImportError:
        tomllib = None

try:
    import tomlkit
    TOMLKIT_AVAILABLE = True
except ImportError:
    TOMLKIT_AVAILABLE = False
    tomlkit = None


class TomlHandler:
    """Gestionnaire TOML avec support optimisé lecture/écriture"""
    
    @staticmethod
    def load(path: Union[str, Path]) -> Dict[str, Any]:
        """Charge un fichier TOML"""
        path = Path(path)
        
        if not path.exists():
            raise FileNotFoundError(f"Fichier TOML introuvable: {path}")
        
        try:
            return TomlHandler._load_with_tomllib(path)
        except Exception as e:
            logger.warning(f"Échec lecture avec tomllib: {e}")
            return TomlHandler._load_with_fallback(path)
    
    @staticmethod
    def loads(content: str) -> Dict[str, Any]:
        """Parse une chaîne TOML"""
        try:
            if tomllib:
                return tomllib.loads(content)
            else:
                return TomlHandler._parse_basic_toml(content)
        except Exception as e:
            logger.error(f"Erreur parsing TOML: {e}")
            raise ValueError(f"Contenu TOML invalide: {e}")
    
    @staticmethod
    def dump(data: Dict[str, Any], path: Union[str, Path]) -> None:
        """Sauvegarde données vers fichier TOML"""
        path = Path(path)
        
        try:
            if TOMLKIT_AVAILABLE:
                TomlHandler._dump_with_tomlkit(data, path)
            else:
                TomlHandler._dump_basic(data, path)
        except Exception as e:
            logger.error(f"Erreur sauvegarde TOML: {e}")
            raise
    
    @staticmethod
    def dumps(data: Dict[str, Any]) -> str:
        """Convertit données vers chaîne TOML"""
        try:
            if TOMLKIT_AVAILABLE:
                doc = tomlkit.document()
                TomlHandler._dict_to_tomlkit(data, doc)
                return doc.as_string()
            else:
                return TomlHandler._dump_basic_string(data)
        except Exception as e:
            logger.error(f"Erreur conversion TOML: {e}")
            raise ValueError(f"Impossible de convertir en TOML: {e}")
    
    # Méthodes privées
    
    @staticmethod
    def _load_with_tomllib(path: Path) -> Dict[str, Any]:
        """Lecture avec tomllib/tomli"""
        if not tomllib:
            raise ImportError("tomllib/tomli non disponible")
        
        with open(path, 'rb') as f:
            return tomllib.load(f)
    
    @staticmethod
    def _load_with_fallback(path: Path) -> Dict[str, Any]:
        """Lecture fallback basique"""
        with open(path, 'r', encoding='utf-8') as f:
            content = f.read()
        return TomlHandler._parse_basic_toml(content)
    
    @staticmethod
    def _dump_with_tomlkit(data: Dict[str, Any], path: Path) -> None:
        """Sauvegarde avec tomlkit (préserve formatage)"""
        doc = tomlkit.document()
        TomlHandler._dict_to_tomlkit(data, doc)
        
        path.parent.mkdir(parents=True, exist_ok=True)
        with open(path, 'w', encoding='utf-8') as f:
            f.write(doc.as_string())
    
    @staticmethod
    def _dump_basic(data: Dict[str, Any], path: Path) -> None:
        """Sauvegarde basique sans tomlkit"""
        path.parent.mkdir(parents=True, exist_ok=True)
        with open(path, 'w', encoding='utf-8') as f:
            TomlHandler._write_dict_as_toml(data, f, 0)
    
    @staticmethod
    def _dict_to_tomlkit(data: Dict[str, Any], container) -> None:
        """Convertit dict vers structure tomlkit"""
        for key, value in data.items():
            if isinstance(value, dict):
                table = tomlkit.table()
                TomlHandler._dict_to_tomlkit(value, table)
                container[key] = table
            elif isinstance(value, list):
                if value and isinstance(value[0], dict):
                    # Array of tables
                    array = tomlkit.aot()
                    for item in value:
                        table = tomlkit.table()
                        TomlHandler._dict_to_tomlkit(item, table)
                        array.append(table)
                    container[key] = array
                else:
                    container[key] = value
            else:
                container[key] = value
    
    @staticmethod
    def _write_dict_as_toml(data: Dict[str, Any], f: TextIO, indent: int) -> None:
        """Écriture TOML basique"""
        indent_str = "  " * indent
        
        # Valeurs simples d'abord
        for key, value in data.items():
            if not isinstance(value, dict):
                if isinstance(value, str):
                    f.write(f'{indent_str}{key} = "{value}"\n')
                elif isinstance(value, bool):
                    f.write(f'{indent_str}{key} = {str(value).lower()}\n')
                elif isinstance(value, list):
                    if value and isinstance(value[0], str):
                        quoted_values = [f'"{v}"' for v in value]
                        f.write(f'{indent_str}{key} = [{", ".join(quoted_values)}]\n')
                    else:
                        f.write(f'{indent_str}{key} = {value}\n')
                else:
                    f.write(f'{indent_str}{key} = {value}\n')
        
        # Tables ensuite
        for key, value in data.items():
            if isinstance(value, dict):
                if indent == 0:
                    f.write(f'\n[{key}]\n')
                else:
                    f.write(f'\n[{".".join([""] * indent)}{key}]\n')
                TomlHandler._write_dict_as_toml(value, f, indent + 1)
    
    @staticmethod
    def _dump_basic_string(data: Dict[str, Any]) -> str:
        """Conversion basique vers string TOML"""
        lines = []
        
        # Valeurs simples
        for key, value in data.items():
            if not isinstance(value, dict):
                if isinstance(value, str):
                    lines.append(f'{key} = "{value}"')
                elif isinstance(value, bool):
                    lines.append(f'{key} = {str(value).lower()}')
                elif isinstance(value, list):
                    if value and isinstance(value[0], str):
                        quoted_values = [f'"{v}"' for v in value]
                        lines.append(f'{key} = [{", ".join(quoted_values)}]')
                    else:
                        lines.append(f'{key} = {value}')
                else:
                    lines.append(f'{key} = {value}')
        
        # Tables
        for key, value in data.items():
            if isinstance(value, dict):
                lines.append(f'\n[{key}]')
                for subkey, subvalue in value.items():
                    if isinstance(subvalue, str):
                        lines.append(f'{subkey} = "{subvalue}"')
                    elif isinstance(subvalue, bool):
                        lines.append(f'{subkey} = {str(subvalue).lower()}')
                    else:
                        lines.append(f'{subkey} = {subvalue}')
        
        return '\n'.join(lines)
    
    @staticmethod
    def _parse_basic_toml(content: str) -> Dict[str, Any]:
        """Parser TOML basique (fallback)"""
        data = {}
        current_section = data
        section_path = []
        
        for line_num, line in enumerate(content.split('\n'), 1):
            line = line.strip()
            
            # Ignorer commentaires et lignes vides
            if not line or line.startswith('#'):
                continue
            
            # Section [table]
            if line.startswith('[') and line.endswith(']'):
                section_name = line[1:-1].strip()
                section_path = section_name.split('.')
                
                # Créer la structure imbriquée
                current_section = data
                for part in section_path[:-1]:
                    if part not in current_section:
                        current_section[part] = {}
                    current_section = current_section[part]
                
                if section_path[-1] not in current_section:
                    current_section[section_path[-1]] = {}
                current_section = current_section[section_path[-1]]
                continue
            
            # Paires clé = valeur
            if '=' in line:
                key, value = line.split('=', 1)
                key = key.strip()
                value = value.strip()
                
                # Parsing basique des valeurs
                if value.startswith('"') and value.endswith('"'):
                    # String
                    current_section[key] = value[1:-1]
                elif value.lower() in ('true', 'false'):
                    # Boolean
                    current_section[key] = value.lower() == 'true'
                elif value.startswith('[') and value.endswith(']'):
                    # Array
                    array_content = value[1:-1].strip()
                    if array_content:
                        items = [item.strip().strip('"') for item in array_content.split(',')]
                        current_section[key] = items
                    else:
                        current_section[key] = []
                elif value.isdigit():
                    # Integer
                    current_section[key] = int(value)
                elif '.' in value and value.replace('.', '').isdigit():
                    # Float
                    current_section[key] = float(value)
                else:
                    # String sans quotes
                    current_section[key] = value
        
        return data


"""
Tests d'intégration de création et gestion des templates
"""

import pytest
from pathlib import Path
import json

class TestTemplateCreation:
    """Tests de création complète de templates"""
    
    def test_create_web_app_template(self, env_manager, tmp_path):
        """Test création template application web"""
        template_service = env_manager.template_service
        
        # Création projet depuis template web
        create_result = template_service.create_from_template(
            template_name="webapp",
            project_path=tmp_path / "my_webapp",
            project_params={
                "project_name": "my-web-app",
                "description": "Application web de test",
                "author": "Test Developer",
                "license": "MIT"
            }
        )
        
        assert create_result.success
        
        project_path = tmp_path / "my_webapp"
        assert project_path.exists()
        assert (project_path / "pyproject.toml").exists()
        
        # Vérification contenu pyproject.toml
        pyproject_content = (project_path / "pyproject.toml").read_text()
        assert "my-web-app" in pyproject_content
        assert "flask" in pyproject_content.lower() or "fastapi" in pyproject_content.lower()
        
        # Création environnement depuis template
        env_result = env_manager.create_from_pyproject(
            pyproject_path=project_path / "pyproject.toml",
            env_name="webapp_env"
        )
        assert env_result.success
        
        # Vérification packages web installés
        env_info = env_result.environment
        package_names = [pkg.name.lower() for pkg in env_info.packages]
        web_packages = ["flask", "fastapi", "django", "requests"]
        assert any(web_pkg in package_names for web_pkg in web_packages)
    
    def test_create_data_science_template(self, env_manager, tmp_path):
        """Test création template data science"""
        template_service = env_manager.template_service
        
        # Création projet data science
        create_result = template_service.create_from_template(
            template_name="datascience",
            project_path=tmp_path / "ml_project",
            project_params={
                "project_name": "ml-analysis",
                "description": "Projet d'analyse ML",
                "author": "Data Scientist"
            }
        )
        
        assert create_result.success
        
        project_path = tmp_path / "ml_project"
        pyproject_content = (project_path / "pyproject.toml").read_text()
        
        # Vérification dépendances data science
        ds_packages = ["pandas", "numpy", "scikit-learn", "matplotlib"]
        for package in ds_packages:
            assert package in pyproject_content
        
        # Test environnement data science
        env_result = env_manager.create_from_pyproject(
            pyproject_path=project_path / "pyproject.toml",
            env_name="datascience_env"
        )
        assert env_result.success
        
        env_info = env_result.environment
        package_names = [pkg.name.lower() for pkg in env_info.packages]
        assert "pandas" in package_names
        assert "numpy" in package_names
    
    def test_create_cli_tool_template(self, env_manager, tmp_path):
        """Test création template outil CLI"""
        template_service = env_manager.template_service
        
        # Création outil CLI
        create_result = template_service.create_from_template(
            template_name="cli",
            project_path=tmp_path / "cli_tool",
            project_params={
                "project_name": "my-cli-tool",
                "description": "Outil CLI de test",
                "entry_point": "mycli"
            }
        )
        
        assert create_result.success
        
        project_path = tmp_path / "cli_tool"
        pyproject_content = (project_path / "pyproject.toml").read_text()
        
        # Vérification configuration CLI
        assert "click" in pyproject_content or "typer" in pyproject_content
        assert "console_scripts" in pyproject_content or "scripts" in pyproject_content
        assert "mycli" in pyproject_content
        
        # Structure CLI typique
        assert (project_path / "src").exists() or (project_path / "my_cli_tool").exists()
    
    def test_custom_template_creation(self, env_manager, tmp_path):
        """Test création template personnalisé"""
        template_service = env_manager.template_service
        
        # Définition template personnalisé
        custom_template = {
            "name": "custom_microservice",
            "description": "Template microservice personnalisé",
            "files": {
                "pyproject.toml": '''[project]
name = "{{project_name}}"
version = "0.1.0"
description = "{{description}}"
dependencies = [
    "fastapi>=0.68.0",
    "uvicorn>=0.15.0",
    "pydantic>=1.8.0"
]

[project.optional-dependencies]
dev = ["pytest>=6.0", "httpx>=0.24.0"]
''',
                "main.py": '''from fastapi import FastAPI

app = FastAPI(title="{{project_name}}")

@app.get("/")
async def root():
    return {"message": "Hello from {{project_name}}"}
''',
                "Dockerfile": '''FROM python:3.11-slim
WORKDIR /app
COPY . .
RUN pip install -e .
CMD ["uvicorn", "main:app", "--host", "0.0.0.0", "--port", "8000"]
'''
            }
        }
        
        # Enregistrement template
        register_result = template_service.register_template(custom_template)
        assert register_result.success
        
        # Utilisation template personnalisé
        create_result = template_service.create_from_template(
            template_name="custom_microservice",
            project_path=tmp_path / "my_microservice",
            project_params={
                "project_name": "my-api",
                "description": "API microservice"
            }
        )
        
        assert create_result.success
        
        project_path = tmp_path / "my_microservice"
        assert (project_path / "main.py").exists()
        assert (project_path / "Dockerfile").exists()
        
        # Vérification substitution paramètres
        main_content = (project_path / "main.py").read_text()
        assert "my-api" in main_content
        assert "{{project_name}}" not in main_content
    
    def test_template_with_environment_creation(self, env_manager, tmp_path):
        """Test création template avec environnement associé"""
        template_service = env_manager.template_service
        
        # Création projet avec environnement automatique
        create_result = template_service.create_project_with_environment(
            template_name="webapp",
            project_path=tmp_path / "webapp_with_env",
            env_name="webapp_auto",
            project_params={
                "project_name": "auto-webapp",
                "description": "Webapp avec env auto"
            }
        )
        
        assert create_result.success
        assert create_result.environment is not None
        assert create_result.environment.name == "webapp_auto"
        
        # Vérification projet et environnement
        project_path = tmp_path / "webapp_with_env"
        assert project_path.exists()
        
        env_info = env_manager.get_environment_info("webapp_auto")
        assert env_info is not None
        assert len(env_info.packages) > 0
    
    def test_template_list_and_management(self, env_manager):
        """Test listage et gestion des templates"""
        template_service = env_manager.template_service
        
        # Listage templates disponibles
        templates_list = template_service.list_templates()
        assert templates_list.success
        assert len(templates_list.templates) > 0
        
        # Vérification templates built-in
        template_names = [t.name for t in templates_list.templates]
        builtin_templates = ["webapp", "datascience", "cli"]
        
        for builtin in builtin_templates:
            assert builtin in template_names
        
        # Informations détaillées template
        webapp_info = template_service.get_template_info("webapp")
        assert webapp_info.success
        assert webapp_info.template.name == "webapp"
        assert webapp_info.template.description
        assert len(webapp_info.template.dependencies) > 0


#!/usr/bin/env python3
import sys
import subprocess

print("Checking GestVenv installation...")
print("-" * 50)

# Test 1: Import direct
try:
    from gestvenv.__version__ import __version__
    print(f"✓ Direct import successful: {__version__}")
except Exception as e:
    print(f"✗ Direct import failed: {e}")

# Test 2: CLI module
try:
    result = subprocess.run([sys.executable, "-m", "gestvenv.cli", "--version"], 
                          capture_output=True, text=True)
    if result.returncode == 0:
        print(f"✓ CLI module works: {result.stdout.strip()}")
    else:
        print(f"✗ CLI module failed: {result.stderr}")
except Exception as e:
    print(f"✗ CLI test failed: {e}")

# Test 3: Installed command
try:
    result = subprocess.run(["gv", "--version"], capture_output=True, text=True)
    if result.returncode == 0:
        print(f"✓ 'gv' command works: {result.stdout.strip()}")
    else:
        print(f"✗ 'gv' command failed: {result.stderr}")
except Exception as e:
    print(f"✗ 'gv' command not found: {e}")

print("-" * 50)
print("Installation tip: pip install -e .")


"""
Module utils de GestVenv v1.1

Ce module contient tous les utilitaires du système :
- TomlHandler : Gestionnaire TOML optimisé
- PyProjectParser : Parser pyproject.toml conforme PEP 621  
- ValidationUtils : Utilitaires de validation
- PathUtils : Utilitaires de gestion des chemins
- SecurityUtils : Utilitaires de sécurité
- PerformanceMonitor : Monitoring de performance
"""

from .toml_handler import TomlHandler
from .pyproject_parser import PyProjectParser
from .validation import ValidationUtils
from .path_utils import PathUtils
from .security import SecurityUtils
from .performance import PerformanceMonitor

__all__ = [
    "TomlHandler",
    "PyProjectParser", 
    "ValidationUtils",
    "PathUtils",
    "SecurityUtils",
    "PerformanceMonitor",
]

# Version du module utils
__version__ = "1.1.0"


"""
Tests d'intégration du système de cache
"""

import pytest
import time
from pathlib import Path

class TestCacheIntegration:
    """Tests d'intégration du cache avec les autres composants"""
    
    def test_cache_package_downloads(self, env_manager):
        """Test mise en cache des téléchargements"""
        cache_service = env_manager.cache_service
        
        # Nettoyage cache initial
        cache_service.clear_cache()
        
        # Première installation (téléchargement)
        result = env_manager.create_environment("cache_test1", python_version="3.11")
        env_info = result.environment
        
        start_time = time.time()
        install_result = env_manager.package_service.install_package(
            env_info, "requests"
        )
        first_install_time = time.time() - start_time
        
        assert install_result.success
        
        # Vérification présence en cache
        cache_info = cache_service.get_cache_info()
        assert cache_info.total_size > 0
        assert cache_info.package_count > 0
        
        # Deuxième installation (depuis cache)
        result2 = env_manager.create_environment("cache_test2", python_version="3.11")
        env_info2 = result2.environment
        
        start_time = time.time()
        install_result2 = env_manager.package_service.install_package(
            env_info2, "requests"
        )
        second_install_time = time.time() - start_time
        
        assert install_result2.success
        # Deuxième installation doit être plus rapide (cache hit)
        assert second_install_time < first_install_time * 0.8
    
    def test_cache_offline_mode(self, env_manager):
        """Test mode hors ligne avec cache"""
        cache_service = env_manager.cache_service
        
        # Installation avec connexion
        result = env_manager.create_environment("offline_prep", python_version="3.11")
        env_info = result.environment
        
        install_result = env_manager.package_service.install_package(
            env_info, "click"
        )
        assert install_result.success
        
        # Activation mode hors ligne
        cache_service.enable_offline_mode()
        
        try:
            # Création nouvel environnement en mode hors ligne
            result2 = env_manager.create_environment("offline_test", python_version="3.11")
            env_info2 = result2.environment
            
            # Installation depuis cache
            install_result2 = env_manager.package_service.install_package(
                env_info2, "click"
            )
            assert install_result2.success
            
        finally:
            cache_service.disable_offline_mode()
    
    def test_cache_cleanup_and_maintenance(self, env_manager):
        """Test nettoyage et maintenance du cache"""
        cache_service = env_manager.cache_service
        
        # Installation plusieurs packages pour remplir cache
        result = env_manager.create_environment("cleanup_test", python_version="3.11")
        env_info = result.environment
        
        packages = ["requests", "click", "pytest", "black"]
        for package in packages:
            env_manager.package_service.install_package(env_info, package)
        
        # Vérification cache rempli
        cache_info_before = cache_service.get_cache_info()
        assert cache_info_before.package_count >= 4
        
        # Nettoyage partiel (anciens packages)
        cleanup_result = cache_service.cleanup_old_packages(days=0)
        assert cleanup_result.success
        
        # Vérification cache partiellement nettoyé
        cache_info_after = cache_service.get_cache_info()
        assert cache_info_after.total_size <= cache_info_before.total_size
    
    def test_cache_backend_integration(self, env_manager):
        """Test intégration cache avec différents backends"""
        cache_service = env_manager.cache_service
        backend_manager = env_manager.backend_manager
        
        # Test avec backend pip
        if backend_manager.backends['pip'].available:
            result = env_manager.create_environment(
                "cache_pip", 
                python_version="3.11",
                backend="pip"
            )
            env_info = result.environment
            
            install_result = env_manager.package_service.install_package(
                env_info, "requests"
            )
            assert install_result.success
            
            cache_info_pip = cache_service.get_cache_info()
            assert cache_info_pip.total_size > 0
        
        # Test avec backend uv (si disponible)
        if backend_manager.backends.get('uv', {}).get('available', False):
            result = env_manager.create_environment(
                "cache_uv",
                python_version="3.11", 
                backend="uv"
            )
            env_info = result.environment
            
            install_result = env_manager.package_service.install_package(
                env_info, "click"
            )
            assert install_result.success
            
            cache_info_uv = cache_service.get_cache_info()
            assert cache_info_uv.total_size > cache_info_pip.total_size
    
    def test_cache_corruption_recovery(self, env_manager):
        """Test récupération après corruption du cache"""
        cache_service = env_manager.cache_service
        
        # Installation normale
        result = env_manager.create_environment("corruption_test", python_version="3.11")
        env_info = result.environment
        
        install_result = env_manager.package_service.install_package(
            env_info, "requests"
        )
        assert install_result.success
        
        # Simulation corruption cache (suppression fichiers)
        cache_dir = cache_service.cache_path
        if cache_dir.exists():
            import shutil
            shutil.rmtree(cache_dir)
        
        # Tentative utilisation avec cache corrompu
        result2 = env_manager.create_environment("recovery_test", python_version="3.11")
        env_info2 = result2.environment
        
        # Doit récupérer automatiquement
        install_result2 = env_manager.package_service.install_package(
            env_info2, "requests"
        )
        assert install_result2.success
        
        # Vérification cache reconstruit
        cache_info = cache_service.get_cache_info()
        assert cache_info.package_count > 0


"""
Gestionnaire principal d'environnements pour GestVenv v1.1
"""

import json
import os
import shutil
import time
from datetime import datetime
from pathlib import Path
from typing import Dict, List, Optional, Any

from .models import (
    EnvironmentInfo,
    EnvironmentResult,
    ActivationResult,
    SyncResult,
    ExportResult,
    DiagnosticReport,
    BackendType,
    SourceFileType,
    EnvironmentHealth,
    ExportFormat,
    PyProjectInfo
)
from .config_manager import ConfigManager
from .exceptions import (
    EnvironmentError,
    EnvironmentNotFoundError,
    EnvironmentExistsError,
    ValidationError,
    BackendError
)


class EnvironmentManager:
    """Gestionnaire principal des environnements virtuels"""
    
    def __init__(self, config_manager: Optional[ConfigManager] = None):
        self.config_manager = config_manager or ConfigManager()
        self._service_container = None
        
    @property
    def backend_manager(self):
        """Backend manager lazy loading"""
        if not hasattr(self, '_backend_manager'):
            from ..backends import BackendManager
            self._backend_manager = BackendManager(self.config_manager.config)
        return self._backend_manager
    
    @property
    def package_service(self):
        """Package service lazy loading"""
        if not hasattr(self, '_package_service'):
            from ..services import PackageService
            self._package_service = PackageService(self.backend_manager, self.cache_service)
        return self._package_service
    
    @property
    def cache_service(self):
        """Cache service lazy loading"""
        if not hasattr(self, '_cache_service'):
            from ..services import CacheService
            self._cache_service = CacheService(self.config_manager.config)
        return self._cache_service
    
    @property
    def migration_service(self):
        """Migration service lazy loading"""
        if not hasattr(self, '_migration_service'):
            from ..services import MigrationService
            self._migration_service = MigrationService()
        return self._migration_service
    
    @property
    def diagnostic_service(self):
        """Diagnostic service lazy loading"""
        if not hasattr(self, '_diagnostic_service'):
            from ..services import DiagnosticService
            self._diagnostic_service = DiagnosticService(self)
        return self._diagnostic_service
    
    @property
    def system_service(self):
        """System service lazy loading"""
        if not hasattr(self, '_system_service'):
            from ..services import SystemService
            self._system_service = SystemService()
        return self._system_service
    
    def create_environment(
        self,
        name: str,
        python_version: Optional[str] = None,
        backend: str = "auto",
        initial_packages: Optional[List[str]] = None,
        custom_path: Optional[Path] = None,
        **options
    ) -> EnvironmentResult:
        """Crée un nouvel environnement virtuel"""
        start_time = time.time()
        
        try:
            # Validation du nom
            self._validate_environment_name(name)
            
            # Vérification existence
            if self._environment_exists(name):
                return EnvironmentResult(
                    success=False,
                    message=f"Environnement '{name}' existe déjà",
                    execution_time=time.time() - start_time
                )
            
            # Préparation paramètres
            python_version = python_version or self.config_manager.config.default_python_version
            env_path = custom_path or self._get_environment_path(name)
            
            # Validation version Python
            if not self.system_service.validate_python_version(python_version):
                return EnvironmentResult(
                    success=False,
                    message=f"Version Python invalide: {python_version}",
                    execution_time=time.time() - start_time
                )
            
            # Sélection backend
            backend_instance = self.backend_manager.get_backend(backend)
            
            # Création environnement
            env_path.mkdir(parents=True, exist_ok=True)
            
            success = backend_instance.create_environment(env_path, python_version)
            if not success:
                # Nettoyage en cas d'échec
                if env_path.exists():
                    shutil.rmtree(env_path, ignore_errors=True)
                return EnvironmentResult(
                    success=False,
                    message="Échec création environnement virtuel",
                    execution_time=time.time() - start_time
                )
            
            # Création métadonnées
            env_info = EnvironmentInfo(
                name=name,
                path=env_path,
                python_version=python_version,
                backend_type=BackendType(backend_instance.name.lower()),
                health=EnvironmentHealth.HEALTHY,
                created_at=datetime.now(),
                updated_at=datetime.now(),
                last_used=datetime.now()
            )
            
            # Installation packages initiaux
            warnings = []
            if initial_packages:
                for package in initial_packages:
                    install_result = self.package_service.install_package(env_info, package)
                    if not install_result.success:
                        warnings.append(f"Échec installation {package}: {install_result.message}")
                
                # Mise à jour liste packages
                env_info.packages = backend_instance.list_packages(env_path)
                env_info.updated_at = datetime.now()
            
            # Sauvegarde métadonnées
            self._save_environment_metadata(env_info)
            
            return EnvironmentResult(
                success=True,
                message=f"Environnement '{name}' créé avec succès",
                environment=env_info,
                warnings=warnings,
                execution_time=time.time() - start_time
            )
            
        except Exception as e:
            # Nettoyage en cas d'erreur
            env_path = custom_path or self._get_environment_path(name)
            if env_path.exists():
                shutil.rmtree(env_path, ignore_errors=True)
            
            return EnvironmentResult(
                success=False,
                message=f"Erreur création environnement: {e}",
                execution_time=time.time() - start_time
            )
    
    def create_from_pyproject(
        self,
        pyproject_path: Path,
        env_name: Optional[str] = None,
        groups: Optional[List[str]] = None,
        **options
    ) -> EnvironmentResult:
        """Crée un environnement depuis pyproject.toml"""
        start_time = time.time()
        
        try:
            # Parsing pyproject.toml
            from ..utils import PyProjectParser
            pyproject_info = PyProjectParser.parse_pyproject_toml(pyproject_path)
            
            # Nom environnement
            env_name = env_name or pyproject_info.name or pyproject_path.parent.name
            
            # Création environnement de base
            backend = options.get('backend', 'auto')
            python_version = pyproject_info.requires_python or options.get('python_version')
            
            result = self.create_environment(
                name=env_name,
                python_version=python_version,
                backend=backend,
                **options
            )
            
            if not result.success:
                return result
            
            env_info = result.environment
            
            # Mise à jour avec informations pyproject
            env_info.pyproject_info = pyproject_info
            env_info.source_file_type = SourceFileType.PYPROJECT_TOML
            
            # Extraction dépendances
            dependencies = pyproject_info.extract_dependencies(groups)
            
            # Installation dépendances
            warnings = result.warnings.copy()
            if dependencies:
                for dep in dependencies:
                    install_result = self.package_service.install_package(env_info, dep)
                    if not install_result.success:
                        warnings.append(f"Échec installation {dep}: {install_result.message}")
                
                # Mise à jour packages
                backend_instance = self.backend_manager.get_backend(backend, env_info)
                env_info.packages = backend_instance.list_packages(env_info.path)
            
            # Gestion groupes dépendances
            if pyproject_info.optional_dependencies:
                env_info.dependency_groups = pyproject_info.optional_dependencies
            
            # Génération fichier lock si supporté
            backend_instance = self.backend_manager.get_backend(backend, env_info)
            if hasattr(backend_instance, 'create_lock_file'):
                try:
                    lock_path = backend_instance.create_lock_file(pyproject_path)
                    if lock_path:
                        env_info.lock_file_path = lock_path
                except Exception as e:
                    warnings.append(f"Échec génération lock file: {e}")
            
            env_info.updated_at = datetime.now()
            self._save_environment_metadata(env_info)
            
            return EnvironmentResult(
                success=True,
                message=f"Environnement '{env_name}' créé depuis pyproject.toml",
                environment=env_info,
                warnings=warnings,
                execution_time=time.time() - start_time
            )
            
        except Exception as e:
            return EnvironmentResult(
                success=False,
                message=f"Erreur création depuis pyproject.toml: {e}",
                execution_time=time.time() - start_time
            )
    
    def activate_environment(self, name: str) -> ActivationResult:
        """Active un environnement"""
        try:
            env_info = self.get_environment_info(name)
            if not env_info:
                return ActivationResult(
                    success=False,
                    message=f"Environnement '{name}' introuvable",
                    activation_script=Path(),
                    activation_command=""
                )
            
            # Désactivation environnement actuel
            self.deactivate_environment()
            
            # Génération script activation
            activation_script = self.system_service.get_activation_script(env_info.path)
            
            # Commande d'activation selon OS
            if os.name == 'nt':  # Windows
                activation_command = f"call {activation_script}"
            else:  # Unix/Linux/macOS
                activation_command = f"source {activation_script}"
            
            # Variables d'environnement
            env_vars = {
                'VIRTUAL_ENV': str(env_info.path),
                'VIRTUAL_ENV_PROMPT': f"({name})",
                'PATH': f"{env_info.path / 'bin'}:{os.environ.get('PATH', '')}"
            }
            
            # Marquage comme actif
            env_info.is_active = True
            env_info.last_used = datetime.now()
            self._save_environment_metadata(env_info)
            
            return ActivationResult(
                success=True,
                message=f"Environnement '{name}' activé",
                activation_script=activation_script,
                activation_command=activation_command,
                environment_variables=env_vars
            )
            
        except Exception as e:
            return ActivationResult(
                success=False,
                message=f"Erreur activation: {e}",
                activation_script=Path(),
                activation_command=""
            )
    
    def deactivate_environment(self) -> bool:
        """Désactive l'environnement actuel"""
        try:
            # Marquage tous environnements comme inactifs
            for env_info in self.list_environments():
                if env_info.is_active:
                    env_info.is_active = False
                    self._save_environment_metadata(env_info)
            return True
        except Exception:
            return False
    
    def delete_environment(self, name: str, force: bool = False) -> bool:
        """Supprime un environnement"""
        try:
            env_info = self.get_environment_info(name)
            if not env_info:
                raise EnvironmentNotFoundError(f"Environnement '{name}' introuvable")
            
            # Vérification si actif
            if env_info.is_active and not force:
                raise EnvironmentError(
                    f"Environnement '{name}' est actif. Utilisez --force pour forcer la suppression"
                )
            
            # Sauvegarde avant suppression
            backup_path = self._backup_environment(env_info)
            
            # Suppression répertoire
            if env_info.path.exists():
                shutil.rmtree(env_info.path)
            
            # Suppression métadonnées
            metadata_path = self._get_metadata_path(name)
            if metadata_path.exists():
                metadata_path.unlink()
            
            return True
            
        except Exception as e:
            raise EnvironmentError(f"Erreur suppression environnement: {e}")
    
    def list_environments(self, **filters) -> List[EnvironmentInfo]:
        """Liste les environnements avec filtres optionnels"""
        environments = []
        envs_path = self.config_manager.get_environments_path()
        
        if not envs_path.exists():
            return environments
        
        for env_dir in envs_path.iterdir():
            if env_dir.is_dir():
                env_info = self._load_environment_metadata(env_dir.name)
                if env_info and self._match_filters(env_info, filters):
                    environments.append(env_info)
        
        return sorted(environments, key=lambda x: x.last_used, reverse=True)
    
    def sync_environment(self, name: str) -> SyncResult:
        """Synchronise un environnement avec son pyproject.toml"""
        start_time = time.time()
        
        try:
            env_info = self.get_environment_info(name)
            if not env_info:
                return SyncResult(
                    success=False,
                    message=f"Environnement '{name}' introuvable",
                    execution_time=time.time() - start_time
                )
            
            if not env_info.pyproject_info:
                return SyncResult(
                    success=False,
                    message="Aucun pyproject.toml associé",
                    execution_time=time.time() - start_time
                )
            
            # Synchronisation avec service packages
            sync_result = self.package_service.sync_environment(env_info)
            sync_result.execution_time = time.time() - start_time
            
            if sync_result.success:
                env_info.updated_at = datetime.now()
                self._save_environment_metadata(env_info)
            
            return sync_result
            
        except Exception as e:
            return SyncResult(
                success=False,
                message=f"Erreur synchronisation: {e}",
                execution_time=time.time() - start_time
            )
    
    def clone_environment(self, source: str, target: str) -> EnvironmentResult:
        """Clone un environnement existant"""
        try:
            source_env = self.get_environment_info(source)
            if not source_env:
                raise EnvironmentNotFoundError(f"Environnement source '{source}' introuvable")
            
            # Création nouvel environnement
            result = self.create_environment(
                name=target,
                python_version=source_env.python_version,
                backend=source_env.backend_type.value
            )
            
            if not result.success:
                return result
            
            # Clonage packages
            target_env = result.environment
            warnings = result.warnings.copy()
            
            for package in source_env.packages:
                install_result = self.package_service.install_package(
                    target_env, 
                    package.get_install_command()
                )
                if not install_result.success:
                    warnings.append(f"Échec clonage {package.name}: {install_result.message}")
            
            # Copie métadonnées
            target_env.pyproject_info = source_env.pyproject_info
            target_env.dependency_groups = source_env.dependency_groups.copy()
            target_env.source_file_type = source_env.source_file_type
            
            # Mise à jour packages
            backend = self.backend_manager.get_backend(target_env.backend_type.value, target_env)
            target_env.packages = backend.list_packages(target_env.path)
            target_env.updated_at = datetime.now()
            
            self._save_environment_metadata(target_env)
            
            return EnvironmentResult(
                success=True,
                message=f"Environnement '{target}' cloné depuis '{source}'",
                environment=target_env,
                warnings=warnings
            )
            
        except Exception as e:
            return EnvironmentResult(
                success=False,
                message=f"Erreur clonage: {e}"
            )
    
    def export_environment(self, name: str, format: ExportFormat) -> ExportResult:
        """Exporte un environnement"""
        try:
            env_info = self.get_environment_info(name)
            if not env_info:
                return ExportResult(
                    success=False,
                    message=f"Environnement '{name}' introuvable",
                    output_path=Path(),
                    format=format,
                    items_exported=0
                )
            
            # Génération chemin de sortie
            output_path = Path.cwd() / f"{name}_export.{format.value}"
            
            if format == ExportFormat.JSON:
                # Export JSON complet
                export_data = env_info.to_dict()
                with open(output_path, 'w', encoding='utf-8') as f:
                    json.dump(export_data, f, indent=2, default=str)
                items_exported = 1
                
            elif format == ExportFormat.REQUIREMENTS:
                # Export requirements.txt
                lines = [pkg.get_install_command() for pkg in env_info.packages]
                output_path = output_path.with_suffix('.txt')
                output_path.write_text('\n'.join(lines), encoding='utf-8')
                items_exported = len(lines)
                
            elif format == ExportFormat.PYPROJECT:
                # Export pyproject.toml
                if env_info.pyproject_info:
                    from ..utils import TomlHandler
                    pyproject_data = {
                        'project': {
                            'name': env_info.pyproject_info.name,
                            'version': env_info.pyproject_info.version,
                            'dependencies': env_info.pyproject_info.dependencies,
                            'optional-dependencies': env_info.pyproject_info.optional_dependencies
                        }
                    }
                    output_path = output_path.with_suffix('.toml')
                    TomlHandler.dump(pyproject_data, output_path)
                    items_exported = 1
                else:
                    return ExportResult(
                        success=False,
                        message="Aucune information pyproject.toml disponible",
                        output_path=Path(),
                        format=format,
                        items_exported=0
                    )
            
            return ExportResult(
                success=True,
                message=f"Environnement exporté vers {output_path}",
                output_path=output_path,
                format=format,
                items_exported=items_exported
            )
            
        except Exception as e:
            return ExportResult(
                success=False,
                message=f"Erreur export: {e}",
                output_path=Path(),
                format=format,
                items_exported=0
            )
    
    def import_environment(self, source: Path, **options) -> EnvironmentResult:
        """Importe un environnement"""
        try:
            if not source.exists():
                return EnvironmentResult(
                    success=False,
                    message=f"Fichier source introuvable: {source}"
                )
            
            env_name = options.get('name') or source.stem
            
            if source.suffix == '.json':
                # Import depuis JSON
                with open(source, 'r', encoding='utf-8') as f:
                    env_data = json.load(f)
                
                env_info = EnvironmentInfo.from_dict(env_data)
                env_info.name = env_name
                env_info.path = self._get_environment_path(env_name)
                
                # Recréation environnement
                result = self.create_environment(
                    name=env_name,
                    python_version=env_info.python_version,
                    backend=env_info.backend_type.value
                )
                
                if result.success:
                    # Restauration packages
                    target_env = result.environment
                    for package in env_info.packages:
                        self.package_service.install_package(
                            target_env, 
                            package.get_install_command()
                        )
                
                return result
            
            else:
                return EnvironmentResult(
                    success=False,
                    message=f"Format non supporté: {source.suffix}"
                )
                
        except Exception as e:
            return EnvironmentResult(
                success=False,
                message=f"Erreur import: {e}"
            )
    
    def doctor_environment(self, name: Optional[str] = None) -> DiagnosticReport:
        """Diagnostic d'un environnement ou du système complet"""
        if name:
            return self.diagnostic_service.diagnose_environment(name)
        else:
            return self.diagnostic_service.run_full_diagnostic()
    
    def get_environment_info(self, name: str) -> Optional[EnvironmentInfo]:
        """Récupère les informations d'un environnement"""
        return self._load_environment_metadata(name)
    
    def auto_migrate_if_needed(self) -> bool:
        """Migration automatique si nécessaire"""
        try:
            return self.migration_service.auto_migrate_if_needed()
        except Exception:
            return False
    
    # Méthodes privées
    
    def _validate_environment_name(self, name: str) -> None:
        """Valide un nom d'environnement"""
        from ..utils import ValidationUtils
        if not ValidationUtils.validate_environment_name(name):
            raise ValidationError(f"Nom d'environnement invalide: {name}")
    
    def _environment_exists(self, name: str) -> bool:
        """Vérifie si un environnement existe"""
        env_path = self._get_environment_path(name)
        return env_path.exists()
    
    def _get_environment_path(self, name: str) -> Path:
        """Chemin d'un environnement"""
        return self.config_manager.get_environments_path() / name
    
    def _get_metadata_path(self, name: str) -> Path:
        """Chemin du fichier métadonnées"""
        return self._get_environment_path(name) / ".gestvenv-metadata.json"
    
    def _save_environment_metadata(self, env_info: EnvironmentInfo) -> None:
        """Sauvegarde les métadonnées d'un environnement"""
        metadata_path = self._get_metadata_path(env_info.name)
        metadata_path.parent.mkdir(parents=True, exist_ok=True)
        
        with open(metadata_path, 'w', encoding='utf-8') as f:
            json.dump(env_info.to_dict(), f, indent=2, default=str)
    
    def _load_environment_metadata(self, name: str) -> Optional[EnvironmentInfo]:
        """Charge les métadonnées d'un environnement"""
        metadata_path = self._get_metadata_path(name)
        
        if not metadata_path.exists():
            # Tentative détection environnement existant sans métadonnées
            env_path = self._get_environment_path(name)
            if env_path.exists():
                return self._detect_existing_environment(name, env_path)
            return None
        
        try:
            with open(metadata_path, 'r', encoding='utf-8') as f:
                data = json.load(f)
            return EnvironmentInfo.from_dict(data)
        except Exception:
            return None
    
    def _detect_existing_environment(self, name: str, env_path: Path) -> Optional[EnvironmentInfo]:
        """Détecte un environnement existant sans métadonnées"""
        try:
            # Détection version Python
            python_version = self.system_service.detect_python_version(env_path)
            
            # Création info basique
            env_info = EnvironmentInfo(
                name=name,
                path=env_path,
                python_version=python_version,
                health=EnvironmentHealth.UNKNOWN
            )
            
            # Détection packages
            backend = self.backend_manager.get_backend("pip")  # Fallback
            env_info.packages = backend.list_packages(env_path)
            
            # Sauvegarde métadonnées pour usage futur
            self._save_environment_metadata(env_info)
            
            return env_info
        except Exception:
            return None
    
    def _backup_environment(self, env_info: EnvironmentInfo) -> Optional[Path]:
        """Sauvegarde un environnement"""
        try:
            backup_dir = Path.home() / ".gestvenv" / "backups" / "environments"
            backup_dir.mkdir(parents=True, exist_ok=True)
            
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            backup_path = backup_dir / f"{env_info.name}_{timestamp}.json"
            
            with open(backup_path, 'w', encoding='utf-8') as f:
                json.dump(env_info.to_dict(), f, indent=2, default=str)
            
            return backup_path
        except Exception:
            return None
    
    def _match_filters(self, env_info: EnvironmentInfo, filters: Dict[str, Any]) -> bool:
        """Vérifie si un environnement correspond aux filtres"""
        if filters.get('active_only') and not env_info.is_active:
            return False
        
        if filters.get('backend') and env_info.backend_type.value != filters['backend']:
            return False
        
        if filters.get('health') and env_info.health.value != filters['health']:
            return False
        
        return True


"""
Tests de performance pour le parsing de fichiers GestVenv v2.0
"""

import pytest
import time
import tempfile
from pathlib import Path

from gestvenv.utils.toml_handler import TomlHandler
from gestvenv.utils.requirements_parser import RequirementsParser


class TestParsingPerformance:
    """Tests de performance pour le parsing"""
    
    @pytest.fixture
    def temp_dir(self):
        """Cree un repertoire temporaire"""
        with tempfile.TemporaryDirectory() as tmpdir:
            yield Path(tmpdir)
    
    def create_simple_toml(self, path: Path) -> Path:
        """Cree un fichier TOML simple"""
        content = '''
[project]
name = "test-project"
version = "1.0.0"

[project.dependencies]
requests = "^2.28.0"
click = ">=8.0"
'''
        toml_file = path / "pyproject.toml"
        toml_file.write_text(content)
        return toml_file
    
    def create_simple_requirements(self, path: Path) -> Path:
        """Cree un fichier requirements.txt simple"""
        content = '''requests==2.28.0
click>=8.0
pandas>=1.0.0
'''
        req_file = path / "requirements.txt"
        req_file.write_text(content)
        return req_file
    
    @pytest.mark.benchmark
    def test_toml_parsing_performance(self, temp_dir):
        """Test de performance basique pour le parsing TOML"""
        handler = TomlHandler()
        toml_file = self.create_simple_toml(temp_dir)
        
        # Mesurer le parsing
        start = time.perf_counter()
        data = handler.load(toml_file)
        duration = time.perf_counter() - start
        
        # Verifications
        assert data is not None
        assert "project" in data
        assert duration < 0.1, f"Parsing TOML trop lent: {duration}s"
    
    @pytest.mark.benchmark
    def test_requirements_parsing_performance(self, temp_dir):
        """Test de performance basique pour le parsing requirements"""
        parser = RequirementsParser()
        req_file = self.create_simple_requirements(temp_dir)
        
        # Mesurer le parsing
        start = time.perf_counter()
        deps = parser.parse_file(str(req_file))
        duration = time.perf_counter() - start
        
        # Verifications
        assert len(deps) == 3
        assert duration < 0.1, f"Parsing requirements trop lent: {duration}s"


if __name__ == "__main__":
    pytest.main([__file__, "-v"])


"""
Routes API pour la gestion du cache.
"""

from fastapi import APIRouter, HTTPException, BackgroundTasks, Query
from typing import Optional
import logging

from api.models.schemas import CacheInfo, CacheExport, CacheImport, ApiResponse
from api.services.gestvenv_service import GestVenvService
from api.services.operation_service import OperationService
from api.websocket import event_emitter

logger = logging.getLogger(__name__)

router = APIRouter(prefix="/cache")

# Services
gestvenv_service = GestVenvService()
operation_service = OperationService()


@router.get("/info", response_model=CacheInfo)
async def get_cache_info():
    """
    Récupère les informations du cache.
    
    Returns:
        Informations du cache
    """
    try:
        cache_info = await gestvenv_service.get_cache_info()
        
        if not cache_info:
            raise HTTPException(status_code=500, detail="Impossible de récupérer les informations du cache")
        
        return cache_info
        
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Failed to get cache info: {e}")
        raise HTTPException(status_code=500, detail="Erreur lors de la récupération des informations du cache")


@router.post("/clean", response_model=ApiResponse)
async def clean_cache(
    background_tasks: BackgroundTasks,
    older_than: Optional[int] = Query(None, description="Nettoyer les éléments plus anciens que X jours"),
    size_limit: Optional[str] = Query(None, description="Nettoyer pour atteindre cette taille max")
):
    """
    Nettoie le cache selon les critères spécifiés.
    
    Args:
        older_than: Age limite en jours
        size_limit: Taille limite (ex: 500MB)
        background_tasks: Tâches en arrière-plan
    
    Returns:
        Réponse API
    """
    try:
        # Créer une opération pour suivre le progrès
        operation_id = operation_service.create_operation("clean_cache")
        
        # Lancer le nettoyage en arrière-plan
        background_tasks.add_task(
            _clean_cache_task,
            operation_id,
            older_than,
            size_limit
        )
        
        return ApiResponse(
            success=True,
            message="Nettoyage du cache démarré",
            data={"operation_id": operation_id}
        )
        
    except Exception as e:
        logger.error(f"Failed to clean cache: {e}")
        raise HTTPException(status_code=500, detail="Erreur lors du nettoyage du cache")


@router.post("/export", response_model=ApiResponse)
async def export_cache(
    export_config: CacheExport,
    background_tasks: BackgroundTasks
):
    """
    Exporte le cache vers un fichier archive.
    
    Args:
        export_config: Configuration d'export
        background_tasks: Tâches en arrière-plan
    
    Returns:
        Réponse API
    """
    try:
        # Créer une opération pour suivre le progrès
        operation_id = operation_service.create_operation("export_cache")
        
        # Lancer l'export en arrière-plan
        background_tasks.add_task(
            _export_cache_task,
            operation_id,
            export_config
        )
        
        return ApiResponse(
            success=True,
            message=f"Export du cache vers '{export_config.output_path}' démarré",
            data={"operation_id": operation_id}
        )
        
    except Exception as e:
        logger.error(f"Failed to export cache: {e}")
        raise HTTPException(status_code=500, detail="Erreur lors de l'export du cache")


@router.post("/import", response_model=ApiResponse)
async def import_cache(
    import_config: CacheImport,
    background_tasks: BackgroundTasks
):
    """
    Importe un cache depuis un fichier archive.
    
    Args:
        import_config: Configuration d'import
        background_tasks: Tâches en arrière-plan
    
    Returns:
        Réponse API
    """
    try:
        # Créer une opération pour suivre le progrès
        operation_id = operation_service.create_operation("import_cache")
        
        # Lancer l'import en arrière-plan
        background_tasks.add_task(
            _import_cache_task,
            operation_id,
            import_config
        )
        
        return ApiResponse(
            success=True,
            message=f"Import du cache depuis '{import_config.source_path}' démarré",
            data={"operation_id": operation_id}
        )
        
    except Exception as e:
        logger.error(f"Failed to import cache: {e}")
        raise HTTPException(status_code=500, detail="Erreur lors de l'import du cache")


# ===== Tâches en arrière-plan =====

async def _clean_cache_task(operation_id: str, older_than: Optional[int], size_limit: Optional[str]):
    """Tâche de nettoyage du cache."""
    try:
        await operation_service.run_operation(
            operation_id,
            gestvenv_service.clean_cache,
            older_than,
            size_limit
        )
        
        # Émettre un événement WebSocket pour la mise à jour du cache
        cache_info = await gestvenv_service.get_cache_info()
        if cache_info:
            await event_emitter.emit_cache_updated(cache_info.model_dump())
        
    except Exception as e:
        operation_service.update_operation(operation_id, error=str(e))


async def _export_cache_task(operation_id: str, export_config: CacheExport):
    """Tâche d'export du cache."""
    try:
        # TODO: Implémenter l'export via le service GestVenv
        operation_service.update_operation(
            operation_id,
            progress=100.0,
            message="Export terminé",
            result={"exported_to": export_config.output_path}
        )
        
    except Exception as e:
        operation_service.update_operation(operation_id, error=str(e))


async def _import_cache_task(operation_id: str, import_config: CacheImport):
    """Tâche d'import du cache."""
    try:
        # TODO: Implémenter l'import via le service GestVenv
        operation_service.update_operation(
            operation_id,
            progress=100.0,
            message="Import terminé",
            result={"imported_from": import_config.source_path}
        )
        
        # Émettre un événement WebSocket pour la mise à jour du cache
        cache_info = await gestvenv_service.get_cache_info()
        if cache_info:
            await event_emitter.emit_cache_updated(cache_info.model_dump())
        
    except Exception as e:
        operation_service.update_operation(operation_id, error=str(e))


"""
Modèles de données pour les environnements éphémères
"""

import uuid
from datetime import datetime
from dataclasses import dataclass, field
from enum import Enum
from pathlib import Path
from typing import Dict, Optional, List, Any

from ..models import BackendType as Backend


class EphemeralStatus(Enum):
    """État d'un environnement éphémère"""
    PENDING = "pending"
    CREATING = "creating"
    READY = "ready"
    RUNNING = "running"
    CLEANING_UP = "cleaning_up"
    DESTROYED = "destroyed"
    FAILED = "failed"


class IsolationLevel(Enum):
    """Niveau d'isolation de sécurité"""
    PROCESS = "process"        # Isolation par processus (défaut)
    NAMESPACE = "namespace"    # Isolation par namespaces Linux
    CONTAINER = "container"    # Isolation par container Docker/Podman
    CHROOT = "chroot"         # Isolation par chroot jail


class StorageBackend(Enum):
    """Backend de stockage pour environnements éphémères"""
    DISK = "disk"         # Stockage disque standard
    TMPFS = "tmpfs"       # Stockage en tmpfs (RAM)
    MEMORY = "memory"     # Stockage en mémoire pure


class SecurityMode(Enum):
    """Mode de sécurité"""
    PERMISSIVE = "permissive"  # Accès réseau et système autorisé
    RESTRICTED = "restricted"   # Accès limité (défaut)
    SANDBOXED = "sandboxed"    # Isolation maximale


@dataclass
class ResourceLimits:
    """Limites de ressources pour un environnement"""
    max_memory: Optional[int] = None  # MB
    max_disk: Optional[int] = None    # MB
    max_processes: int = 10
    max_cpu_percent: Optional[float] = None
    network_access: bool = True


@dataclass
class EphemeralEnvironment:
    """Modèle d'environnement éphémère"""
    # Identification
    id: str = field(default_factory=lambda: str(uuid.uuid4()))
    name: Optional[str] = None
    
    # Configuration de base
    backend: Backend = Backend.UV
    python_version: str = "3.11"
    
    # Configuration de cycle de vie
    ttl: Optional[int] = None        # Time To Live en secondes
    max_idle_time: int = 300         # 5 minutes par défaut
    auto_cleanup: bool = True
    
    # Ressources et sécurité
    resource_limits: ResourceLimits = field(default_factory=ResourceLimits)
    isolation_level: IsolationLevel = IsolationLevel.PROCESS
    security_mode: SecurityMode = SecurityMode.RESTRICTED
    
    # État runtime
    status: EphemeralStatus = EphemeralStatus.PENDING
    created_at: datetime = field(default_factory=datetime.now)
    last_activity: datetime = field(default_factory=datetime.now)
    cleanup_scheduled: bool = False
    
    # Chemins et processus
    storage_path: Optional[Path] = None
    venv_path: Optional[Path] = None
    pid: Optional[int] = None
    container_id: Optional[str] = None
    
    # Isolation (nouveaux attributs)
    isolation_script: Optional[str] = None
    chroot_path: Optional[Path] = None
    
    # Métadonnées
    tags: Dict[str, str] = field(default_factory=dict)
    parent_session: Optional[str] = None
    packages: List[str] = field(default_factory=list)
    
    # Statistiques
    creation_time: Optional[float] = None
    cleanup_time: Optional[float] = None
    peak_memory_mb: Optional[float] = None
    peak_disk_mb: Optional[float] = None

    def __post_init__(self):
        """Post-initialisation"""
        if self.name is None:
            self.name = f"ephemeral-{int(self.created_at.timestamp())}"
    
    @property
    def is_active(self) -> bool:
        """Vérifie si l'environnement est actif"""
        return self.status in [
            EphemeralStatus.READY,
            EphemeralStatus.RUNNING
        ]
    
    @property
    def age_seconds(self) -> int:
        """Âge de l'environnement en secondes"""
        return int((datetime.now() - self.created_at).total_seconds())
    
    @property
    def idle_seconds(self) -> int:
        """Temps d'inactivité en secondes"""
        return int((datetime.now() - self.last_activity).total_seconds())
    
    def update_activity(self):
        """Met à jour le timestamp de dernière activité"""
        self.last_activity = datetime.now()
    
    def is_expired(self) -> bool:
        """Vérifie si l'environnement a expiré"""
        if self.ttl is None:
            return False
        return self.age_seconds > self.ttl
    
    def is_idle_expired(self) -> bool:
        """Vérifie si l'environnement est inactif trop longtemps"""
        return self.idle_seconds > self.max_idle_time


@dataclass 
class EphemeralConfig:
    """Configuration globale pour environnements éphémères"""
    # Limites globales
    default_ttl: int = 3600              # 1 heure par défaut
    max_concurrent: int = 50             # Maximum d'environnements simultanés
    max_total_memory_mb: int = 8192      # 8GB maximum total
    max_total_disk_mb: int = 20480       # 20GB maximum total
    
    # Nettoyage
    cleanup_interval: int = 60           # Nettoyage toutes les minutes
    force_cleanup_after: int = 7200      # Nettoyage forcé après 2h
    
    # Stockage
    storage_backend: StorageBackend = StorageBackend.TMPFS
    base_storage_path: Optional[Path] = None
    
    # Sécurité par défaut
    default_security_mode: SecurityMode = SecurityMode.RESTRICTED
    default_isolation_level: IsolationLevel = IsolationLevel.PROCESS
    
    # Monitoring
    enable_monitoring: bool = True
    monitoring_interval: int = 5         # Surveillance toutes les 5 secondes
    
    # Performance
    enable_preallocation: bool = True    # Pré-allocation d'environnements
    pool_size: int = 3                   # Taille du pool de pré-allocation
    enable_template_cache: bool = True   # Cache des templates
    
    def __post_init__(self):
        """Post-initialisation de la configuration"""
        if self.base_storage_path is None:
            if self.storage_backend == StorageBackend.TMPFS:
                self.base_storage_path = Path("/tmp/gestvenv-ephemeral")
            elif self.storage_backend == StorageBackend.MEMORY:
                self.base_storage_path = Path("/dev/shm/gestvenv-ephemeral")
            else:
                self.base_storage_path = Path.home() / ".cache/gestvenv/ephemeral"


@dataclass
class OperationResult:
    """Résultat d'une opération dans un environnement éphémère"""
    returncode: int
    stdout: str
    stderr: str
    duration: float
    command: str
    success: bool = field(init=False)
    
    def __post_init__(self):
        self.success = self.returncode == 0


@dataclass
class ResourceUsage:
    """Usage des ressources d'un environnement"""
    memory_mb: float
    disk_mb: float
    cpu_percent: float
    active_processes: int
    network_bytes_sent: int = 0
    network_bytes_recv: int = 0
    timestamp: datetime = field(default_factory=datetime.now)


@dataclass
class CleanupReason:
    """Raison du nettoyage d'un environnement"""
    reason: str
    triggered_by: str
    forced: bool = False
    error: Optional[Exception] = None
    timestamp: datetime = field(default_factory=datetime.now)


"""
Template Django
"""

from typing import Dict, List, Any
from ..base_template import ProjectTemplate, TemplateFile


class DjangoTemplate(ProjectTemplate):
    """Template pour projets Django"""
    
    def __init__(self):
        super().__init__(
            name="django",
            description="Projet Django avec configuration moderne",
            category="web",
            dependencies=[
                "django>=4.2.0",
                "django-environ>=0.10.0",
                "psycopg2-binary>=2.9.0"
            ],
            dev_dependencies=[
                "pytest>=7.0.0",
                "pytest-django>=4.5.0",
                "black>=23.0.0",
                "isort>=5.12.0",
                "django-debug-toolbar>=4.0.0"
            ]
        )
        
    def get_files(self) -> List[TemplateFile]:
        return [
            TemplateFile(
                path="{{package_name}}/settings.py",
                content='''"""Configuration Django pour {{project_name}}"""

from pathlib import Path
import environ

env = environ.Env(
    DEBUG=(bool, False)
)

BASE_DIR = Path(__file__).resolve().parent.parent

environ.Env.read_env(BASE_DIR / '.env')

DEBUG = env('DEBUG')
SECRET_KEY = env('SECRET_KEY', default='django-insecure-dev-key')

ALLOWED_HOSTS = env.list('ALLOWED_HOSTS', default=['localhost', '127.0.0.1'])

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'core',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = '{{package_name}}.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR / 'templates'],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}

LANGUAGE_CODE = 'fr-fr'
TIME_ZONE = 'Europe/Paris'
USE_I18N = True
USE_TZ = True

STATIC_URL = '/static/'
STATICFILES_DIRS = [BASE_DIR / 'static']

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
'''
            ),
            TemplateFile(
                path="{{package_name}}/urls.py",
                content='''"""URLs principales de {{project_name}}"""

from django.contrib import admin
from django.urls import path, include

urlpatterns = [
    path('admin/', admin.site.urls),
    path('', include('core.urls')),
]
'''
            ),
            TemplateFile(
                path="core/__init__.py",
                content="",
                is_template=False
            ),
            TemplateFile(
                path="core/models.py",
                content='''"""Modèles de l'application core"""

from django.db import models


class Item(models.Model):
    """Modèle d'exemple"""
    name = models.CharField(max_length=100)
    description = models.TextField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    
    def __str__(self):
        return self.name
    
    class Meta:
        verbose_name = "Item"
        verbose_name_plural = "Items"
        ordering = ['-created_at']
'''
            ),
            TemplateFile(
                path="core/views.py",
                content='''"""Vues de l'application core"""

from django.shortcuts import render
from django.http import JsonResponse
from .models import Item


def index(request):
    """Vue d'accueil"""
    return render(request, 'core/index.html', {
        'title': '{{project_name}}'
    })


def api_items(request):
    """API des items"""
    items = Item.objects.all()
    data = [
        {
            'id': item.id,
            'name': item.name,
            'description': item.description
        }
        for item in items
    ]
    return JsonResponse({'items': data})
'''
            ),
            TemplateFile(
                path="core/urls.py",
                content='''"""URLs de l'application core"""

from django.urls import path
from . import views

urlpatterns = [
    path('', views.index, name='index'),
    path('api/items/', views.api_items, name='api_items'),
]
'''
            ),
            TemplateFile(
                path="manage.py",
                content='''#!/usr/bin/env python
"""Utilitaire de gestion Django pour {{project_name}}"""

import os
import sys

if __name__ == '__main__':
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', '{{package_name}}.settings')
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed?"
        ) from exc
    execute_from_command_line(sys.argv)
'''
            ),
            TemplateFile(
                path=".env.example",
                content='''# Configuration pour {{project_name}}
DEBUG=True
SECRET_KEY=your-secret-key-here
ALLOWED_HOSTS=localhost,127.0.0.1
''',
                is_template=False
            )
        ]
        
    def get_pyproject_config(self) -> Dict[str, Any]:
        return {
            "project": {
                "name": "{{project_name}}",
                "version": "{{version}}",
                "description": "{{description}}",
                "authors": [{"name": "{{author}}", "email": "{{email}}"}],
                "license": {"text": "{{license}}"},
                "readme": "README.md",
                "requires-python": ">=3.8",
                "dependencies": self.dependencies,
                "optional-dependencies": {
                    "dev": self.dev_dependencies,
                    "prod": ["gunicorn>=20.0.0", "whitenoise>=6.0.0"]
                }
            },
            "build-system": {
                "requires": ["setuptools>=61.0", "wheel"],
                "build-backend": "setuptools.build_meta"
            }
        }


"""
Monitoring et suivi des ressources pour environnements éphémères
"""

import asyncio
import logging
import psutil
import time
from pathlib import Path
from typing import Dict, Optional, List

from .models import (
    EphemeralEnvironment,
    ResourceUsage,
    EphemeralStatus
)
from .exceptions import EphemeralException

logger = logging.getLogger(__name__)


class ResourceTracker:
    """Suivi des ressources en temps réel"""
    
    def __init__(self, manager):
        self.manager = manager
        self.monitoring_task: Optional[asyncio.Task] = None
        self.resource_history: Dict[str, List[ResourceUsage]] = {}
        self._shutdown = False
    
    async def start(self):
        """Démarrage du monitoring"""
        if self.monitoring_task is None:
            logger.info("Starting resource monitoring")
            self.monitoring_task = asyncio.create_task(self._monitoring_loop())
    
    async def stop(self):
        """Arrêt du monitoring"""
        self._shutdown = True
        if self.monitoring_task:
            self.monitoring_task.cancel()
            try:
                await self.monitoring_task
            except asyncio.CancelledError:
                pass
            self.monitoring_task = None
        logger.info("Resource monitoring stopped")
    
    async def start_monitoring(self, env: EphemeralEnvironment):
        """Démarrage du monitoring pour un environnement spécifique"""
        if env.id not in self.resource_history:
            self.resource_history[env.id] = []
        logger.debug(f"Started monitoring for environment {env.id}")
    
    async def stop_monitoring(self, env_id: str):
        """Arrêt du monitoring pour un environnement"""
        if env_id in self.resource_history:
            # Conservation de l'historique pendant un court moment
            history = self.resource_history[env_id]
            if history:
                logger.debug(f"Stopped monitoring for {env_id}, peak memory: {max(r.memory_mb for r in history):.1f}MB")
            del self.resource_history[env_id]
    
    async def get_current_usage(self, env: EphemeralEnvironment) -> Optional[ResourceUsage]:
        """Récupération de l'usage actuel d'un environnement"""
        try:
            return await self._measure_environment_resources(env)
        except Exception as e:
            logger.warning(f"Failed to get usage for {env.id}: {e}")
            return None
    
    async def get_resource_history(self, env_id: str) -> List[ResourceUsage]:
        """Récupération de l'historique des ressources"""
        return self.resource_history.get(env_id, [])
    
    async def _monitoring_loop(self):
        """Boucle principale de monitoring"""
        while not self._shutdown:
            try:
                # Monitoring de tous les environnements actifs
                environments = await self.manager.list_environments()
                
                for env in environments:
                    if env.is_active and env.id in self.resource_history:
                        usage = await self._measure_environment_resources(env)
                        if usage:
                            await self._record_usage(env, usage)
                
                # Nettoyage de l'historique ancien
                await self._cleanup_old_history()
                
                # Attente avant la prochaine mesure
                await asyncio.sleep(self.manager.config.monitoring_interval)
                
            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.error(f"Monitoring loop error: {e}")
                await asyncio.sleep(30)  # Attente plus longue en cas d'erreur
    
    async def _measure_environment_resources(self, env: EphemeralEnvironment) -> Optional[ResourceUsage]:
        """Mesure des ressources d'un environnement"""
        try:
            # Mesure de l'usage disque
            disk_usage_mb = 0.0
            if env.storage_path and env.storage_path.exists():
                disk_usage_mb = await self._get_directory_size(env.storage_path)
            
            # Mesure de l'usage mémoire et CPU
            memory_mb = 0.0
            cpu_percent = 0.0
            active_processes = 0
            
            if env.pid:
                try:
                    # Processus principal
                    main_process = psutil.Process(env.pid)
                    if main_process.is_running():
                        memory_mb += main_process.memory_info().rss / (1024 * 1024)
                        cpu_percent += main_process.cpu_percent()
                        active_processes += 1
                        
                        # Processus enfants
                        for child in main_process.children(recursive=True):
                            try:
                                if child.is_running():
                                    memory_mb += child.memory_info().rss / (1024 * 1024)
                                    cpu_percent += child.cpu_percent()
                                    active_processes += 1
                            except (psutil.NoSuchProcess, psutil.AccessDenied):
                                continue
                                
                except (psutil.NoSuchProcess, psutil.AccessDenied):
                    # Processus terminé ou accès refusé
                    pass
            
            return ResourceUsage(
                memory_mb=memory_mb,
                disk_mb=disk_usage_mb,
                cpu_percent=cpu_percent,
                active_processes=active_processes
            )
            
        except Exception as e:
            logger.warning(f"Failed to measure resources for {env.id}: {e}")
            return None
    
    async def _get_directory_size(self, path: Path) -> float:
        """Calcul de la taille d'un répertoire en MB"""
        try:
            def get_size():
                total_size = 0
                for dirpath, dirnames, filenames in path.walk():
                    for filename in filenames:
                        try:
                            file_path = dirpath / filename
                            total_size += file_path.stat().st_size
                        except (OSError, FileNotFoundError):
                            continue
                return total_size / (1024 * 1024)  # Conversion en MB
            
            # Exécution dans un thread pour éviter le blocage
            loop = asyncio.get_event_loop()
            return await loop.run_in_executor(None, get_size)
            
        except Exception:
            return 0.0
    
    async def _record_usage(self, env: EphemeralEnvironment, usage: ResourceUsage):
        """Enregistrement d'une mesure d'usage"""
        # Ajout à l'historique
        if env.id in self.resource_history:
            self.resource_history[env.id].append(usage)
            
            # Limitation de la taille de l'historique (30 dernières mesures)
            if len(self.resource_history[env.id]) > 30:
                self.resource_history[env.id] = self.resource_history[env.id][-30:]
        
        # Mise à jour des pics dans l'environnement
        if env.peak_memory_mb is None or usage.memory_mb > env.peak_memory_mb:
            env.peak_memory_mb = usage.memory_mb
        
        if env.peak_disk_mb is None or usage.disk_mb > env.peak_disk_mb:
            env.peak_disk_mb = usage.disk_mb
        
        # Vérification des limites
        await self._check_resource_limits(env, usage)
    
    async def _check_resource_limits(self, env: EphemeralEnvironment, usage: ResourceUsage):
        """Vérification des limites de ressources"""
        limits = env.resource_limits
        
        # Vérification mémoire
        if limits.max_memory and usage.memory_mb > limits.max_memory:
            logger.warning(
                f"Environment {env.id} exceeded memory limit: "
                f"{usage.memory_mb:.1f}MB > {limits.max_memory}MB"
            )
            # TODO: Déclencher action de limite (warning, throttling, cleanup)
        
        # Vérification disque
        if limits.max_disk and usage.disk_mb > limits.max_disk:
            logger.warning(
                f"Environment {env.id} exceeded disk limit: "
                f"{usage.disk_mb:.1f}MB > {limits.max_disk}MB"
            )
        
        # Vérification nombre de processus
        if usage.active_processes > limits.max_processes:
            logger.warning(
                f"Environment {env.id} exceeded process limit: "
                f"{usage.active_processes} > {limits.max_processes}"
            )
    
    async def _cleanup_old_history(self):
        """Nettoyage de l'historique ancien"""
        current_time = time.time()
        
        for env_id, history in list(self.resource_history.items()):
            # Suppression des mesures de plus de 5 minutes
            cutoff_time = current_time - 300
            
            filtered_history = [
                usage for usage in history
                if usage.timestamp.timestamp() > cutoff_time
            ]
            
            if filtered_history:
                self.resource_history[env_id] = filtered_history
            else:
                # Environnement probablement terminé
                del self.resource_history[env_id]


class PerformanceMonitor:
    """Monitoring des performances système globales"""
    
    def __init__(self):
        self.system_metrics: Dict[str, float] = {}
    
    async def get_system_load(self) -> Dict[str, float]:
        """Récupération de la charge système"""
        try:
            # Charge CPU
            cpu_percent = psutil.cpu_percent(interval=1)
            
            # Utilisation mémoire
            memory = psutil.virtual_memory()
            memory_percent = memory.percent
            memory_available_gb = memory.available / (1024**3)
            
            # Utilisation disque
            disk = psutil.disk_usage('/')
            disk_percent = (disk.used / disk.total) * 100
            disk_available_gb = disk.free / (1024**3)
            
            return {
                "cpu_percent": cpu_percent,
                "memory_percent": memory_percent,
                "memory_available_gb": memory_available_gb,
                "disk_percent": disk_percent,
                "disk_available_gb": disk_available_gb,
                "load_average": psutil.getloadavg()[0] if hasattr(psutil, 'getloadavg') else 0.0
            }
            
        except Exception as e:
            logger.error(f"Failed to get system metrics: {e}")
            return {}
    
    async def check_system_health(self) -> bool:
        """Vérification de la santé du système"""
        try:
            metrics = await self.get_system_load()
            
            # Critères de santé
            if metrics.get("cpu_percent", 0) > 90:
                logger.warning("High CPU usage detected")
                return False
            
            if metrics.get("memory_percent", 0) > 95:
                logger.warning("High memory usage detected")
                return False
            
            if metrics.get("disk_percent", 0) > 95:
                logger.warning("High disk usage detected")
                return False
            
            return True
            
        except Exception as e:
            logger.error(f"System health check failed: {e}")
            return False


"""WebSocket routes for real-time IDE communication."""

from fastapi import APIRouter, WebSocket, WebSocketDisconnect, Depends, Query
from typing import Optional
import uuid
import logging

from ..websocket.manager import manager
from ..websocket.handlers import WebSocketHandler
from ..services.environment_service import EnvironmentService
from ..services.package_service import PackageService
from ..core.dependencies import get_environment_service, get_package_service

logger = logging.getLogger(__name__)

router = APIRouter()


@router.websocket("/ws/ide/{client_id}")
async def websocket_endpoint(
    websocket: WebSocket,
    client_id: str,
    env_service: EnvironmentService = Depends(get_environment_service),
    pkg_service: PackageService = Depends(get_package_service)
):
    """
    WebSocket endpoint for IDE real-time communication.
    
    Client ID should be unique per VS Code instance.
    """
    # Extract metadata from query params
    metadata = {
        "user_agent": websocket.headers.get("user-agent", ""),
        "vscode_version": websocket.query_params.get("vscode_version"),
        "extension_version": websocket.query_params.get("extension_version"),
        "workspace": websocket.query_params.get("workspace")
    }
    
    # Connect client
    await manager.connect(websocket, client_id, metadata)
    
    # Create handler
    handler = WebSocketHandler(env_service, pkg_service)
    
    try:
        while True:
            # Receive message
            data = await websocket.receive_json()
            logger.debug(f"Received message from {client_id}: {data}")
            
            # Handle message
            await handler.handle_message(client_id, data)
            
    except WebSocketDisconnect:
        logger.info(f"Client {client_id} disconnected normally")
        manager.disconnect(client_id)
    except Exception as e:
        logger.error(f"WebSocket error for client {client_id}: {e}")
        manager.disconnect(client_id)


@router.get("/ws/status")
async def websocket_status():
    """Get WebSocket server status."""
    return {
        "connected_clients": manager.get_client_count(),
        "clients": list(manager.active_connections.keys()),
        "environment_subscriptions": {
            client_id: list(envs)
            for client_id, envs in manager.client_environments.items()
        }
    }


"""
Tests de performance pour le systeme de cache GestVenv v2.0
"""

import pytest
import time
import tempfile
import shutil
from pathlib import Path
from typing import Dict, Any

from gestvenv.services.cache_service import CacheService
from gestvenv.core.config_manager import ConfigManager


class TestCachePerformance:
    """Tests de performance pour le systeme de cache"""
    
    @pytest.fixture
    def temp_cache_dir(self):
        """Cree un repertoire temporaire pour le cache"""
        temp_dir = tempfile.mkdtemp()
        yield Path(temp_dir)
        shutil.rmtree(temp_dir, ignore_errors=True)
    
    @pytest.fixture
    def config_manager(self, temp_cache_dir):
        """Configuration manager pour les tests"""
        config = ConfigManager()
        config._config['cache_directory'] = str(temp_cache_dir)
        config._config['cache_enabled'] = True
        return config
    
    @pytest.fixture
    def cache_service(self, config_manager):
        """Service de cache pour les tests"""
        return CacheService(config_manager)
    
    def create_fake_package(self, name: str, size_mb: int) -> bytes:
        """Cree un faux package pour les tests"""
        # Genere des donnees pseudo-aleatoires
        data = (name * 1024).encode() * (size_mb * 1024)
        return data[:size_mb * 1024 * 1024]
    
    @pytest.mark.benchmark
    def test_cache_initialization_performance(self, config_manager):
        """Test de performance pour l'initialisation du cache"""
        start = time.perf_counter()
        cache = CacheService(config_manager)
        duration = time.perf_counter() - start
        
        assert duration < 0.1, f"Initialisation trop lente: {duration}s"
    
    @pytest.mark.benchmark
    def test_cache_write_performance(self, cache_service):
        """Test de performance pour l'ecriture dans le cache"""
        # Test avec differentes tailles
        test_sizes = [
            ("small", 1),    # 1 MB
            ("medium", 10),  # 10 MB
        ]
        
        results = {}
        
        for name, size_mb in test_sizes:
            package_data = self.create_fake_package(name, size_mb)
            package_info = {
                "name": f"test_{name}",
                "version": "1.0.0",
                "size": len(package_data),
                "python_version": "3.11"
            }
            
            start = time.perf_counter()
            # Simule l'ajout au cache (methode simplifiee)
            cache_path = cache_service.cache_dir / f"test_{name}-1.0.0.whl"
            cache_path.write_bytes(package_data)
            duration = time.perf_counter() - start
            
            results[name] = {
                "size_mb": size_mb,
                "duration": duration,
                "mb_per_sec": size_mb / duration if duration > 0 else float('inf')
            }
        
        # Verifications
        assert results["small"]["mb_per_sec"] > 10, \
            "Ecriture trop lente pour petits packages"
        assert results["medium"]["mb_per_sec"] > 5, \
            "Ecriture trop lente pour packages moyens"
    
    @pytest.mark.benchmark
    def test_cache_cleanup_performance(self, cache_service):
        """Test de performance pour le nettoyage du cache"""
        # Creer plusieurs fichiers
        num_files = 50
        
        for i in range(num_files):
            file_path = cache_service.cache_dir / f"test_package_{i}.whl"
            file_path.write_bytes(b"dummy data")
        
        # Mesurer le temps de nettoyage
        start = time.perf_counter()
        # Simule un nettoyage
        for file in cache_service.cache_dir.glob("*.whl"):
            file.unlink()
        duration = time.perf_counter() - start
        
        assert duration < 1.0, f"Nettoyage trop lent: {duration}s pour {num_files} fichiers"


if __name__ == "__main__":
    pytest.main([__file__, "-v"])


"""
Service pour la gestion des templates de projet
"""

from typing import List, Dict, Any, Optional
from pathlib import Path


class TemplateService:
    """Service pour gérer les templates de projet"""
    
    def __init__(self):
        self.templates = {
            "django": {
                "name": "Django",
                "description": "Django web application template",
                "packages": ["django", "djangorestframework", "django-cors-headers"]
            },
            "fastapi": {
                "name": "FastAPI",
                "description": "FastAPI REST API template",
                "packages": ["fastapi", "uvicorn", "pydantic"]
            },
            "data-science": {
                "name": "Data Science",
                "description": "Data science project template",
                "packages": ["numpy", "pandas", "matplotlib", "jupyter", "scikit-learn"]
            }
        }
    
    async def list_templates(self) -> List[Dict[str, Any]]:
        """Liste tous les templates disponibles"""
        return [
            {
                "id": key,
                "name": value["name"],
                "description": value["description"],
                "packages_count": len(value["packages"])
            }
            for key, value in self.templates.items()
        ]
    
    async def get_template(self, template_id: str) -> Optional[Dict[str, Any]]:
        """Récupère un template spécifique"""
        if template_id in self.templates:
            template = self.templates[template_id]
            return {
                "id": template_id,
                "name": template["name"],
                "description": template["description"],
                "packages": template["packages"]
            }
        return None
    
    async def create_from_template(self, template_id: str, project_name: str, path: str) -> Dict[str, Any]:
        """Crée un projet à partir d'un template"""
        return {
            "success": True,
            "message": f"Project {project_name} created from template {template_id}",
            "path": path
        }


"""
Routes API pour la gestion des packages.
"""

from fastapi import APIRouter, HTTPException, BackgroundTasks, Query, Path
from typing import List, Optional
import logging

from api.models.schemas import Package, PackageInstall, ApiResponse
from api.services.gestvenv_service import GestVenvService
from api.services.operation_service import OperationService
from api.websocket import event_emitter

logger = logging.getLogger(__name__)

router = APIRouter(prefix="/packages")

# Services
gestvenv_service = GestVenvService()
operation_service = OperationService()


@router.post("/install", response_model=ApiResponse)
async def install_package(
    package_data: PackageInstall,
    background_tasks: BackgroundTasks,
    env_name: str = Query(..., description="Nom de l'environnement")
):
    """
    Installe un package dans un environnement.
    
    Args:
        package_data: Données du package à installer
        env_name: Nom de l'environnement
        background_tasks: Tâches en arrière-plan
    
    Returns:
        Réponse API avec l'ID de l'opération
    """
    try:
        # Créer une opération pour suivre le progrès
        operation_id = operation_service.create_operation("install_package")
        
        # Lancer l'installation en arrière-plan
        background_tasks.add_task(
            _install_package_task,
            operation_id,
            env_name,
            package_data
        )
        
        return ApiResponse(
            success=True,
            message=f"Installation du package '{package_data.name}' démarrée",
            data={"operation_id": operation_id}
        )
        
    except Exception as e:
        logger.error(f"Failed to install package {package_data.name}: {e}")
        raise HTTPException(status_code=500, detail="Erreur lors de l'installation du package")


@router.delete("/{package_name}", response_model=ApiResponse)
async def uninstall_package(
    background_tasks: BackgroundTasks,
    package_name: str = Path(..., description="Nom du package"),
    env_name: str = Query(..., description="Nom de l'environnement")
):
    """
    Désinstalle un package d'un environnement.
    
    Args:
        package_name: Nom du package
        env_name: Nom de l'environnement
        background_tasks: Tâches en arrière-plan
    
    Returns:
        Réponse API
    """
    try:
        # Créer une opération pour suivre le progrès
        operation_id = operation_service.create_operation("uninstall_package")
        
        # Lancer la désinstallation en arrière-plan
        background_tasks.add_task(
            _uninstall_package_task,
            operation_id,
            env_name,
            package_name
        )
        
        return ApiResponse(
            success=True,
            message=f"Désinstallation du package '{package_name}' démarrée",
            data={"operation_id": operation_id}
        )
        
    except Exception as e:
        logger.error(f"Failed to uninstall package {package_name}: {e}")
        raise HTTPException(status_code=500, detail="Erreur lors de la désinstallation")


@router.post("/update", response_model=ApiResponse)
async def update_packages(
    background_tasks: BackgroundTasks,
    env_name: str = Query(..., description="Nom de l'environnement"),
    packages: Optional[List[str]] = Query(None, description="Packages à mettre à jour")
):
    """
    Met à jour des packages dans un environnement.
    
    Args:
        env_name: Nom de l'environnement
        packages: Liste des packages (tous si None)
        background_tasks: Tâches en arrière-plan
    
    Returns:
        Réponse API
    """
    try:
        # Créer une opération pour suivre le progrès
        operation_id = operation_service.create_operation("update_packages")
        
        # Lancer la mise à jour en arrière-plan
        background_tasks.add_task(
            _update_packages_task,
            operation_id,
            env_name,
            packages
        )
        
        message = "Mise à jour de tous les packages démarrée" if not packages else f"Mise à jour de {len(packages)} packages démarrée"
        
        return ApiResponse(
            success=True,
            message=message,
            data={"operation_id": operation_id}
        )
        
    except Exception as e:
        logger.error(f"Failed to update packages in {env_name}: {e}")
        raise HTTPException(status_code=500, detail="Erreur lors de la mise à jour")


# ===== Tâches en arrière-plan =====

async def _install_package_task(operation_id: str, env_name: str, package_data: PackageInstall):
    """Tâche d'installation de package."""
    try:
        await operation_service.run_operation(
            operation_id,
            gestvenv_service.install_package,
            env_name,
            package_data.name,
            package_data.group,
            package_data.editable,
            package_data.upgrade
        )
        
        # Émettre un événement WebSocket
        await event_emitter.emit_package_installed(
            env_name,
            package_data.name,
            package_data.model_dump()
        )
        
    except Exception as e:
        operation_service.update_operation(operation_id, error=str(e))


async def _uninstall_package_task(operation_id: str, env_name: str, package_name: str):
    """Tâche de désinstallation de package."""
    try:
        await operation_service.run_operation(
            operation_id,
            gestvenv_service.uninstall_package,
            env_name,
            package_name
        )
        
    except Exception as e:
        operation_service.update_operation(operation_id, error=str(e))


async def _update_packages_task(operation_id: str, env_name: str, packages: Optional[List[str]]):
    """Tâche de mise à jour de packages."""
    try:
        await operation_service.run_operation(
            operation_id,
            gestvenv_service.update_packages,
            env_name,
            packages
        )
        
    except Exception as e:
        operation_service.update_operation(operation_id, error=str(e))


"""
Services pour GestVenv Web API.
"""

from .gestvenv_service import GestVenvService
from .operation_service import OperationService
from .environment_service import EnvironmentService
from .package_service import PackageService
from .cache_service import CacheService
from .template_service import TemplateService

__all__ = [
    "GestVenvService", 
    "OperationService",
    "EnvironmentService",
    "PackageService",
    "CacheService",
    "TemplateService"
]


"""
Tests d'intégration de la gestion des packages
"""

import pytest
from gestvenv.core.models import PackageInfo, SyncResult

class TestPackageManagement:
    """Tests de gestion complète des packages"""
    
    def test_package_installation_workflow(self, env_manager):
        """Test workflow installation de packages"""
        # Création environnement
        result = env_manager.create_environment("pkg_test", python_version="3.11")
        env_info = result.environment
        
        # Installation package simple
        install_result = env_manager.package_service.install_package(
            env_info, "requests"
        )
        assert install_result.success
        assert install_result.package_info.name == "requests"
        
        # Installation avec version spécifique
        install_result = env_manager.package_service.install_package(
            env_info, "click==8.1.3"
        )
        assert install_result.success
        assert install_result.package_info.version == "8.1.3"
        
        # Installation multiple
        packages = ["pytest", "black", "flake8"]
        install_result = env_manager.package_service.install_packages(
            env_info, packages
        )
        assert install_result.success
        assert len(install_result.packages_info) == 3
    
    def test_package_update_workflow(self, env_manager):
        """Test workflow mise à jour packages"""
        result = env_manager.create_environment("update_test", python_version="3.11")
        env_info = result.environment
        
        # Installation version ancienne
        install_result = env_manager.package_service.install_package(
            env_info, "requests==2.25.0"
        )
        assert install_result.success
        
        # Mise à jour
        update_result = env_manager.package_service.update_package(
            env_info, "requests"
        )
        assert update_result.success
        
        # Vérification version mise à jour
        env_info_updated = env_manager.get_environment_info("update_test")
        requests_pkg = next(
            (pkg for pkg in env_info_updated.packages if pkg.name == "requests"),
            None
        )
        assert requests_pkg
        assert requests_pkg.version != "2.25.0"
    
    def test_package_removal_workflow(self, env_manager):
        """Test workflow suppression packages"""
        result = env_manager.create_environment("remove_test", python_version="3.11")
        env_info = result.environment
        
        # Installation packages
        packages = ["requests", "click", "pytest"]
        env_manager.package_service.install_packages(env_info, packages)
        
        # Suppression package
        remove_result = env_manager.package_service.remove_package(
            env_info, "pytest"
        )
        assert remove_result.success
        
        # Vérification suppression
        env_info_updated = env_manager.get_environment_info("remove_test")
        package_names = [pkg.name for pkg in env_info_updated.packages]
        assert "pytest" not in package_names
        assert "requests" in package_names
        assert "click" in package_names
    
    def test_dependency_resolution(self, env_manager):
        """Test résolution des dépendances"""
        result = env_manager.create_environment("deps_test", python_version="3.11")
        env_info = result.environment
        
        # Installation package avec dépendances
        install_result = env_manager.package_service.install_package(
            env_info, "flask"
        )
        assert install_result.success
        
        # Vérification dépendances installées
        env_info_updated = env_manager.get_environment_info("deps_test")
        package_names = [pkg.name for pkg in env_info_updated.packages]
        
        # Flask a des dépendances comme Werkzeug, Jinja2, etc.
        assert "flask" in package_names
        assert len(package_names) > 1  # Au moins Flask + ses dépendances
    
    def test_package_conflict_resolution(self, env_manager):
        """Test résolution des conflits de versions"""
        result = env_manager.create_environment("conflict_test", python_version="3.11")
        env_info = result.environment
        
        # Installation packages avec versions spécifiques
        env_manager.package_service.install_package(env_info, "requests==2.25.0")
        
        # Tentative installation package nécessitant version différente
        install_result = env_manager.package_service.install_package(
            env_info, "requests==2.28.0"
        )
        
        # Doit soit réussir avec mise à jour, soit échouer proprement
        if install_result.success:
            env_info_updated = env_manager.get_environment_info("conflict_test")
            requests_pkg = next(
                (pkg for pkg in env_info_updated.packages if pkg.name == "requests"),
                None
            )
            assert requests_pkg.version == "2.28.0"
        else:
            assert "conflit" in install_result.message.lower() or "conflict" in install_result.message.lower()
    
    def test_sync_with_pyproject_changes(self, env_manager, tmp_path):
        """Test synchronisation après modification pyproject.toml"""
        # Création pyproject.toml initial
        initial_content = '''[project]
name = "sync-test"
dependencies = ["requests>=2.25.0"]
'''
        pyproject_path = tmp_path / "pyproject.toml"
        pyproject_path.write_text(initial_content)
        
        # Création environnement
        result = env_manager.create_from_pyproject(
            pyproject_path=pyproject_path,
            env_name="sync_test"
        )
        env_info = result.environment
        
        # Modification pyproject.toml
        updated_content = '''[project]
name = "sync-test"
dependencies = ["requests>=2.25.0", "click>=8.0"]
'''
        pyproject_path.write_text(updated_content)
        
        # Synchronisation
        sync_result = env_manager.sync_environment("sync_test")
        assert sync_result.success
        
        # Vérification nouveau package installé
        env_info_updated = env_manager.get_environment_info("sync_test")
        package_names = [pkg.name for pkg in env_info_updated.packages]
        assert "click" in package_names


"""
Configuration de l'application GestVenv Web API.
"""

from pydantic_settings import BaseSettings, SettingsConfigDict
from typing import List
import os


class Settings(BaseSettings):
    """Configuration de l'application."""
    
    # Configuration de base
    APP_NAME: str = "GestVenv Web API"
    VERSION: str = "1.0.0"
    DEBUG: bool = False
    
    # Configuration serveur
    HOST: str = "0.0.0.0"
    PORT: int = 8000
    
    # Configuration CORS - stored as string in env, parsed to list
    ALLOWED_ORIGINS: str = "http://localhost:3000,http://localhost:5173,http://127.0.0.1:3000,http://127.0.0.1:5173"
    
    @property
    def cors_origins(self) -> List[str]:
        """Parse ALLOWED_ORIGINS string to list."""
        return [origin.strip() for origin in self.ALLOWED_ORIGINS.split(',') if origin.strip()]
    
    # Configuration base de données
    DATABASE_URL: str = "sqlite:///./gestvenv_web.db"
    
    # Configuration Redis (optionnel pour cache/sessions)
    REDIS_URL: str = "redis://localhost:6379"
    USE_REDIS: bool = False
    
    # Configuration JWT (pour authentification future)
    SECRET_KEY: str = "your-secret-key-change-in-production"
    ALGORITHM: str = "HS256"
    ACCESS_TOKEN_EXPIRE_MINUTES: int = 30
    
    # Configuration GestVenv
    GESTVENV_CLI_PATH: str = "gestvenv"  # Chemin vers l'exécutable GestVenv
    
    # Configuration fichiers statiques
    SERVE_STATIC_FILES: bool = True
    
    # Configuration logging
    LOG_LEVEL: str = "INFO"
    
    # Configuration WebSocket
    WS_MAX_CONNECTIONS: int = 100
    
    model_config = SettingsConfigDict(
        env_file=".env",
        case_sensitive=True,
        # Ne pas essayer de parser automatiquement comme JSON
        env_parse_none_str="None"
    )


# Instance globale des settings
settings = Settings()


def get_settings() -> Settings:
    """Récupère l'instance des settings."""
    return settings


#!/usr/bin/env python3
"""Fix encoding issues in test files"""

import os
import re

def fix_encoding(file_path):
    """Fix encoding issues in a file"""
    try:
        with open(file_path, 'r', encoding='utf-8') as f:
            content = f.read()
    except UnicodeDecodeError:
        # Try with latin-1 encoding
        with open(file_path, 'r', encoding='latin-1') as f:
            content = f.read()
    
    # Replace common French accented characters
    replacements = {
        'é': 'e',
        'è': 'e',
        'ê': 'e',
        'ë': 'e',
        'à': 'a',
        'â': 'a',
        'ä': 'a',
        'ç': 'c',
        'ù': 'u',
        'û': 'u',
        'ü': 'u',
        'ô': 'o',
        'ö': 'o',
        'î': 'i',
        'ï': 'i',
        'É': 'E',
        'È': 'E',
        'Ê': 'E',
        'Ë': 'E',
        'À': 'A',
        'Â': 'A',
        'Ä': 'A',
        'Ç': 'C',
        'Ù': 'U',
        'Û': 'U',
        'Ü': 'U',
        'Ô': 'O',
        'Ö': 'O',
        'Î': 'I',
        'Ï': 'I'
    }
    
    for old, new in replacements.items():
        content = content.replace(old, new)
    
    # Write back with UTF-8 encoding
    with open(file_path, 'w', encoding='utf-8') as f:
        f.write(content)
    
    print(f"Fixed encoding in {file_path}")

# Fix all performance test files
test_files = [
    "tests/performance/test_backend_performance.py",
    "tests/performance/test_cache_performance.py",
    "tests/performance/test_parsing_performance.py"
]

for file in test_files:
    if os.path.exists(file):
        fix_encoding(file)